#!/usr/bin/env python
"""
crsm.py
Python wrapper for compressed row storage in Lourakis sba code
"""

from __future__ import absolute_import
import ctypes

class Crsm(ctypes.Structure):
    """Sparse matrix represntation using compressed row storage
    Python wrapper for libsba struct sba_crsm.
    nr, nc - rows, columns for the sparse matrix
    nnz - number of nonzero array elements
    val - storage for nonzero array elements, size nnz
    colidx - column indices for nonzero elements, size nnz
    rowptr - locations in val that start a row, size nr+1"""
    _fields_ = [("nr", ctypes.c_int),
                ("nc", ctypes.c_int),
                ("nnz", ctypes.c_int),
                ("val", ctypes.POINTER(ctypes.c_int)),
                ("colidx", ctypes.POINTER(ctypes.c_int)),
                ("rowptr", ctypes.POINTER(ctypes.c_int))]




from __future__ import absolute_import
from sba.version import version as __version__
from sba.cameras import *
from sba.points import *
from sba.projections import *
from sba.options import *
from sba.crsm import *
from sba.errors import *
from sba.info import *
from sba.routines import *
from sba.drivers import *



#!/usr/bin/env/python
"""
cameras.py
For dealing with cameras in sba only
"""


from __future__ import absolute_import
import logging
import numpy as np
import ctypes
import sba.quaternions as quaternions # need this for composition of rotations
from six.moves import range

CAMS_CNP_ALL = 5+5+3+3 # cnp for cameras with distortion - use imag q only
CAMS_CNP_NODIST = 5+3+3 # cnp for cameras without distortion - use imag q only
CAMS_CNP_INTR_FIXED = 3+3 # cnp for cameras with fixed intrinsics

class Cameras(object):
    """
    Cameras for use in sba routines are an array of ncamera rows,
    with cnp columns (number of camera parameters). The camera 
    parameters are (one row for each camera):

    fx cx cy AR s r2 r4 t1 t2 r6 qi qj qk tx ty tz
   
    note q0 is NOT included since q0 = sqrt(1-qi**2-qj**2-qk**2)!!!
    
    where: 
    fx - focal length for x in pixels
    cx, cy - principal point in pixels, typically in middle of image
    AR - aspect ratio, typically 1.
    s - skew, typically 0.
    r2,r4,r6 - radial distortion coeffs according to Bourguet (dimensionless)
    t1,t2 - tangential distortion coeffs (dimensionless)
    q0,qi,qj,qk - unit quaternion rotation of camera (dimensionless)
    tx,ty,tz - translation of camera in real world units

    The class can be asked for the array, the raveled array, 
    the number of cameras (ncameras) and the parameters for one camera (cnp).
    """
    def __init__(self,camarray=None):
        if camarray is None:
            self.__camarray = np.zeros((0,0),dtype=np.double)
        else:
            self.__camarray = np.array(camarray,dtype=np.double)
        self.__ncameras = self.__camarray.shape[0]
        self.__cnp = self.__camarray.shape[1]
        self.__rot0params = np.zeros((self.__ncameras,4),dtype=np.double)

    def _getcamarray(self):
        return self.__camarray
    def _setcamarray(self,newcamarray):
        if newcamarray is None:
            self.__camarray = None
            self.__initrot = None
            self.__ncameras = 0
            self.__cnp = 0
        else:
            self.__camarray = np.array(newcamarray,dtype=np.double)
            self.__ncameras = self.__camarray.shape[0]
            self.__cnp = self.__camarray.shape[1]
    camarray = property(_getcamarray,_setcamarray,doc="""
Camera array for sba
Note that it is of the format for full problem (cnp=5+5+3+3)
fx1 cx1 cy1 ar1 s1 r21 r41 t11 t21 r61 qi1 qj1 qk1 tx1 ty1 tz1

q0 is not carried around because it is not a free, independent variable
q is a unit quaternion so q0 must equal 1-(length of imag part)
""")

    def _getrot0params(self):
        return self.__rot0params
    def _setrot0params(self,newrot0params):
        self.__rot0params = np.array(newrot0params,dtype=np.double)
        assert self.__rot0params.shape == (self.__ncameras,4)
    rot0params = property(_getrot0params,_setrot0params,doc="""
Camera initial rotation for sba as quaternions
q0 qi qj qk
q0 is carried around here. ?!?
""")

    def _getcnp(self):
        return self.__cnp
    cnp = property(_getcnp,doc="cnp = number of parameters for 1 camera")
    
    @property
    def hasFixedIntrinsics(self):
        return self.cnp == CAM_CNP_INTR_FIXED
    
    @property
    def hasDistortion(self):
        return self.cnp == CAM_CNP_ALL

    def _getncameras(self):
        return self.__ncameras
    ncameras = property(_getncameras,doc="ncameras = m = number of cameras")

    def zeroLocalRotationEstimates(self):
        """This is because sba is crazy."""
        A = self.camarray
        A[:,-6:-3]=0.
        self.camarray = A
        
    def combineLocalAndInitialRotation(self,initial):
        """Not sure what this does but Lourakis has it. It is used to
        combine local (optimization result?) rotations and initial rotations
        in cases that allow camera motion."""
        for cam in range(self.ncameras):
            v = self.camarray[cam,-6:-3] # retrieve vector part from self
            v0 = 1-np.sqrt(v[0]**2.+v[1]**2.+v[2]**2) # compute v0

            # turn it into quaternions
            qlocal = quaternions.Quaternion(v0,v[0],v[1],v[2])
            qinitial = quaternions.Quaternion(initial[cam,0],initial[cam,1],
                                              initial[cam,2],initial[cam,3])
            # combine(compose) rotations by quaternion rotation
            qresult = qlocal.mul(qinitial)
            result = qresult.asVector()
            
            # Lourakis likes positive vectors i guess?
            self.camarray[cam,-6:-3] = np.copysign(result[1:],result[0])

    def Aravel(self):
        """
        Use this to ravel the camera array as A when providing to sba
        It will have to be combined with B when providing to sba C routines

        It will provide a one dimensional Numpy array like this:
        fx0 cx0 cy0 AR0 s0 r20 r40 t10 t20 r60 qi0 qj0 qk0 tx0 ty0 tz0...
        fx1 cx1 cy1 AR1 s1 r21 r41 t11 t21 r61 qi1 qj1 qk1 tx1 ty1 tz1...
        etc
        fxN cxN cyN ARN sN r2N r4N t1N t2N r6N q0N qiN qjN qkN txN tyN tzN...

        The length of this array should be ncameras*cnp
        """
        return self.__camarray.ravel()

    def cnpToC(self):
        """Provides cnp as a C int"""
        return ctypes.c_int(self.cnp)

    def ncamerasToC(self):
        """Provides ncameras as a C int"""
        return ctypes.c_int(self.ncameras)

    def __repr__(self):
        """Provide readable representation of Cameras"""
        return "<sba.Cameras() with {0} cameras, each with {1} parameters (q0 not independent)>".format(self.ncameras,self.cnp)

    def _fromTxt(cls,camfile):
        """
        Alternative constructor to read cameras from a text file of the format:

        #comment
        fx cx cy AR s r2 r4 t1 t2 r6 q0 qi qj qk tx ty tz
        
        fx - focal length for x in pixels
        cx, cy - principal point in pixels, typically in middle of image
        AR - aspect ratio, typically 1.
        s - skew, typically 0.
        r2,r4,r6 - radial distortion coeffs according to Bourguet (dimeless)
        t1,t2 - tangential distortion coeffs (dimensionless)
        q0,qi,qj,qk - unit quaternion rotation of camera (dimensionless)
        tx,ty,tz - translation of camera in real world units
        """
        #logging.debug("camerasFromTxt() reading cameras from {0}".format(camfile))
        raw = np.genfromtxt(camfile)
        if raw.shape[1]==17:
            # got a file with distortion coeffs in middle
            # cnp = 5+5+3+3
            camarray = np.hstack((raw[:,0:10],raw[:,-6:])) 
        else:
            # got a file with no distortion coeffs in middle
            # cnp = 5+3+3
            camarray = np.hstack((raw[:,0:5],raw[:,-6:]))
        #print raw,raw.shape
        #print camarray,camarray.shape
        result = Cameras(camarray)
        result.rot0params = raw[:,-7:-3]
        return result
    fromTxt = classmethod(_fromTxt)

    def toTxt(self,camoname):
        """
        Save cameras to a text file of the format:
        fx cx cy AR s r2 r4 t1 t2 r6 q0 qi qj qk tx ty tz
        
        fx - focal length for x in pixels
        cx, cy - principal point in pixels, typically in middle of image
        AR - aspect ratio, typically 1.
        s - skew, typically 0.
        r2,r4,r6 - radial distortion coeffs according to Bourguet (dimeless)
        t1,t2 - tangential distortion coeffs (dimensionless)
        q0,qi,qj,qk - unit quaternion rotation of camera (dimensionless)
        tx,ty,tz - translation of camera in real world units
        """
        #print self.camarray,self.camarray.shape
        outarray = np.zeros((self.ncameras,self.cnp+1),dtype=np.double)

        if self.cnp == 5+5+3+3:
            outarray[:,0:10] = self.camarray[:,0:10]
            outarray[:,-6:] = self.camarray[:,-6:]
        elif self.cnp == 5+3+3:
            outarray[:,0:5] = self.camarray[:,0:5]
            outarray[:,-6:] = self.camarray[:,-6:]
        elif self.cnp == 3+3:
            logging.warning("FIXED INTRINSICS NOT FULLY IMPLEMENTED")
        else:
            logging.warning("HUH????")

        # construct missing q0 real part of unit quaternions
        for cam in range(self.ncameras):
            outarray[cam,-7] = np.sqrt(1-self.camarray[cam,-6]**2.-self.camarray[cam,-5]**2.-self.camarray[cam,-4]**2.)

        # and save it
        np.savetxt(camoname,outarray,"%4.5f")

    def _fromTxtWithIntr(cls,camfile,intrfile):
        """
        Alternative constructor to read cameras from text files of the format:

        #comment for intrfile
        fx cx cy AR s r2 r4 t1 t2 r6 

        #comment for camfile
        q0 qi qj qk tx ty tz
        
        fx - focal length for x in pixels
        cx, cy - principal point in pixels, typically in middle of image
        AR - aspect ratio, typically 1.
        s - skew, typically 0.
        r2,r4,r6 - radial distortion coeffs according to Bourguet (dimeless)
        t1,t2 - tangential distortion coeffs (dimensionless)
        q0,qi,qj,qk - unit quaternion rotation of camera (dimensionless)
        tx,ty,tz - translation of camera in real world units
        """
        #logging.debug("camerasFromTxt() reading cameras from {0}".format(camfile))
        raw = np.genfromtxt(camfile)
        rawi = np.genfromtxt(intrfile)
        intr = np.array([rawi,]*raw.shape()[0]) # copy intrinsics to array

        camarray = np.hstack((intr,raw)) # and stack as before
        result = Cameras(camarray)
        result.rot0params = raw[:,-7:-3]
        return result
    fromTxtWithIntr = classmethod(_fromTxtWithIntr)



    # added for Dylan Ray to use in Python gui version
    def _fromDylan(cls,dylan_array):
        """
        Alternate constructor to read cameras from a numpy array of shape
        (n,17):

        each row is a different camera, the columns are:
        fx cx cy AR s r2 r4 t1 t2 r6 q0 qi qj qk tx ty tz
        
        fx - focal length for x in pixels
        cx, cy - principal point in pixels, typically in middle of image
        AR - aspect ratio, typically 1.
        s - skew, typically 0.
        r2,r4,r6 - radial distortion coeffs according to Bourguet (dimeless)
        t1,t2 - tangential distortion coeffs (dimensionless)
        q0,qi,qj,qk - unit quaternion rotation of camera (dimensionless)
        tx,ty,tz - translation of camera in real world units

        dtype is np.double
        """
        #logging.debug("sba.Cameras.fromDylan() reading cameras from array")
        #logging.warning("this feature not yet tested!")

        try: 
            assert(dylan_array.shape[1]==17)
        except:
            logging.error("bad array passed in sba.Cameras.fromDylan()")

        dylan_array = dylan_array.astype(np.double)
        camarray = np.hstack((dylan_array[:,0:10],dylan_array[:,-6:])) 
        result = Cameras(camarray)
        result.rot0params = dylan_array[:,-7:-3]
        return result
    fromDylan = classmethod(_fromDylan)



    def toDylan(self):
        """
        Returns a numpy array of shape (n,17) containing the cameras:
        each row is a different camera, the columns are:
        fx cx cy AR s r2 r4 t1 t2 r6 q0 qi qj qk tx ty tz
        dtype is np.double
        """
        result = np.array((self.ncameras, 17),dtype=np.double)
        result[:,0:10] = self.camarray[:,0:10]
        result[:,-6:] = self.camarray[:,-6:]

        # construct missing q0 real part of unit quaternions
        for cam in range(self.ncameras):
            result[cam,-7] = np.sqrt(1-self.camarray[cam,-6]**2.
                                     -self.camarray[cam,-5]**2.
                                     -self.camarray[cam,-4]**2.)
        return(result)




#!/usr/bin/env/python
"""
drivers.py
Dennis Evangelista, 2013
"""


from __future__ import absolute_import
from __future__ import print_function
import sba.routines
import sba.options
import sba.info
import logging
import numpy as np
import ctypes
import time
import copy

ERROR = -1

def SparseBundleAdjust(cameras,points,
                       options=None,
                       projection=None, jacobian=None):
    """
    Take Cameras object, Points object and optional Options object
    and performs sparse bundle adjustment using Lourakis' sba C library

    Returns updated newcameras, newpoints, and an Information object
    giving results from the sparse bundle adjustment. 

    Additional options can be specified with an optional Options input.

    Also, Python callable functions can be specified for projection and
    Jacobian; these must conform to the return type and argtypes expected
    by sba in C - see sba.projections for more information.
    """

    logging.debug("sba.SparseBundleAdjust using {0}".format(points))
    logging.debug("sba.SparseBundleAdjust using {0}".format(cameras))

    if options==None:
        options = sba.options.Options.fromInput(cameras,points)

    info = sba.info.Information.fromInput(cameras,points,options)
    logging.debug("sba.SparseBundleAdjust created Information() object")
    newinfo = sba.info.CInfoType()

    routine = sba.routines.whichRoutine(options)
    info.method = routine.__str__
    logging.debug("sba.SparseBundleAdjust using routine {0}".format(routine.__str__))

    if projection==None:
        projection = sba.projections.whichProjection(options)
        logging.debug("sba.SparseBundleAdjust using projection {0}".format(projection.__str__))

    if jacobian==None:
        jacobian = sba.projections.whichJacobian(options)
        logging.debug("sba.SparseBundleAdjust using Jacobian {0}".format(jacobian.__str__))


    # start timer here
    logging.debug("sba.SparseBundleAdjust starting timer and calling C")
    info.t0 = time.clock()

    # dispatcher
    if options.expert:
        if options.motstruct==sba.options.OPTS_MOTSTRUCT:
            cameras.zeroLocalRotationEstimates()
            P = copy.deepcopy(np.hstack((cameras.Aravel(),points.Bravel())))
            newX = points.XtoC() # Have to do this to get back numpy array
            print(("n {0}".format(points.n)))
            print(("m {0}".format(cameras.ncameras)))
            print(("x {0}".format(newX[0:5])))
            print(("p {0}".format(P[0:5])))
            print(("all x {0}".format(newX)))
            print(("length x {0}".format(len(newX))))
            n = routine(ctypes.c_int(points.n), #const int
                        ctypes.c_int(0), #const int
                        ctypes.c_int(cameras.ncameras), # const int
                        ctypes.c_int(0), #const int
                        points.vmaskToC(),
                        P.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
                        ctypes.c_int(options.cnp), # const int
                        ctypes.c_int(options.pnp), # const int
                        newX.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
                        points.covxToC(),
                        ctypes.c_int(options.mnp), # const int
                        sba.FuncProjection(projection), # pointer
                        sba.FjacProjJacobian(jacobian), # pointer
                        options.globsToC(), # void*?
                        options.maxIterToC(), # const
                        options.verboseToC(), # const
                        options.optsToC(), # const
                        newinfo) # pointer, i want it back
        elif options.motstruct==sba.options.OPTS_MOT:
            cameras.zeroLocalRotationEstimates()
            options.ptparams = points.Bravel()
            P = copy.deepcopy(cameras.Aravel())
            newX = points.XtoC()
            n = routine(ctypes.c_int(points.n),
                        ctypes.c_int(cameras.ncameras),
                        ctypes.c_int(0),
                        points.vmaskToC(),
                        P.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
                        ctypes.c_int(options.cnp),
                        newX.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
                        points.covxToC(),
                        ctypes.c_int(options.mnp),
                        sba.FuncProjection(projection),
                        sba.FjacProjJacobian(jacobian),
                        options.globsToC(),
                        options.maxIterToC(),
                        options.verboseToC(),
                        options.optsToC(),
                        newinfo)
        elif options.motstruct==sba.options.OPTS_STRUCT:
            P = copy.deepcopy(points.Bravel())
            options.camparams = cameras.Aravel()
            newX = points.XtoC()
            n = routine(ctypes.c_int(points.n),
                        ctypes.c_int(0),
                        ctypes.c_int(cameras.ncameras),
                        points.vmaskToC(),
                        P.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
#                        ctypes.c_int(options.cnp),
                        ctypes.c_int(options.pnp),
                        newX.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
                        points.covxToC(),
                        ctypes.c_int(options.mnp),
                        sba.FuncProjection(projection),
                        sba.FjacProjJacobian(jacobian),
                        options.globsToC(),
                        options.maxIterToC(),
                        options.verboseToC(),
                        options.optsToC(),
                        newinfo)
        else:
            raise sba.errors.SbaError("Unrecognized sba option")
    else:
        logging.warning("Simple mode untested!")
        if options.motstruct==sba.options.OPTS_MOTSTRUCT:
            cameras.zeroLocalRotationEstimates()
            P = np.hstack((cameras.Aravel(),points.Bravel())).copy()
            newX = points.XtoC() # Have to do this to get back numpy array
            n = routine(ctypes.c_int(points.n), #const int
                        ctypes.c_int(0), #const int
                        ctypes.c_int(cameras.ncameras), # const int
                        ctypes.c_int(0), #const int
                        points.vmaskToC(),
                        P.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
                        ctypes.c_int(options.cnp), # const int
                        ctypes.c_int(options.pnp), # const int
                        newX.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
                        points.covxToC(),
                        ctypes.c_int(options.mnp), # const int
                        sba.FuncProjectionSimple(projection), # pointer
                        sba.FjacProjJacobianSimple(jacobian), # pointer
                        options.globsToC(), # void*?
                        options.maxIterToC(), # const
                        options.verboseToC(), # const
                        options.optsToC(), # const
                        newinfo) # pointer, i want it back
        elif options.motstruct==sba.options.OPTS_MOT:
            cameras.zeroLocalRotationEstimates()
            P = copy.deepcopy(cameras.Aravel())
            options.ptsparams = points.Bravel()
            newX = points.XtoC()
            n = routine(ctypes.c_int(points.n),
                        ctypes.c_int(cameras.ncameras),
                        ctypes.c_int(0),
                        points.vmaskToC(),
                        P.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
                        ctypes.c_int(options.cnp),
                        ctypes.c_int(options.pnp),
                        newX.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
                        points.covxToC(),
                        ctypes.c_int(options.mnp),
                        sba.FuncProjectionSimple(projection),
                        sba.FjacProjJacobianSimple(jacobian),
                        options.globsToC(),
                        options.maxIterToC(),
                        options.verboseToC(),
                        options.optsToC(),
                        newinfo)
        elif options.motstruct==sba.options.OPTS_STRUCT:
            P = copy.deepcopy(points.Bravel())
            options.camparams = cameras.Aravel()
            newX = points.XtoC()
            n = routine(ctypes.c_int(points.n),
                        ctypes.c_int(0),
                        ctypes.c_int(cameras.ncameras),
                        points.vmaskToC(),
                        P.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
                        ctypes.c_int(options.cnp),
                        ctypes.c_int(options.pnp),
                        newX.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
                        points.covxToC(),
                        ctypes.c_int(options.mnp),
                        sba.FuncProjectionSimple(projection),
                        sba.FjacProjJacobianSimple(jacobian),
                        options.globsToC(),
                        options.maxIterToC(),
                        options.verboseToC(),
                        options.optsToC(),
                        newinfo)
        else:
            raise sba.errors.SbaError("Unrecognized sba option")
        pass
    
    # stop timer here
    info.tf = time.clock()
    logging.debug("sba.SparseBundleAdjust returned from C and stopped timer")
    
    if n == ERROR:
        raise sba.errors.SbaError("sparseBundleAdjust() C call returned error {0}".format(n))





    # unpack some stuff
    logging.debug("sba.SparseBundleAdjust unpacking results")
    newcameras = copy.deepcopy(cameras)
    newpoints = copy.deepcopy(points)
    info.fromC(newinfo)
    if options.motstruct==sba.options.OPTS_MOTSTRUCT:
        newcameras.camarray = P[0:options.ncameras*options.cnp].reshape(options.ncameras,options.cnp) # unpack A from P first part
        newcameras.combineLocalAndInitialRotation(options.rot0params) # combine local rotation estimates with initial ones
        newpoints.B = P[options.ncameras*options.cnp:].reshape(newpoints.n,options.pnp) # unpack B from P second part

    elif options.motstruct==sba.options.OPTS_MOT:
        newcameras.camarray = P.reshape(options.ncameras,options.cnp) # unload A camera params only
        newcameras.combineLocalAndInitialRotation(options.rot0params) # combine local rotation estimates with initial ones

    elif options.motstruct==sba.options.OPTS_STRUCT:
        newpoints.B = P.reshape(options.n,options.pnp) # unload B structure params only

    else:
        raise sba.errors.SbaError("Unrecognized option for howto: {0}".format(options.motstruct))
    # refined motion and structure are now in motstruct
    
    logging.debug("sba.SparseBundleAdjust returning results")
    return newcameras,newpoints,info















#!/usr/bin/env python
"""
routines.py
Python wrapper for sba C library by Manolis Lourakis, ver 1.6
Wrapper by Dennis Evangelista, 2013

This part wraps the actual routines, eg sba_motstr_levmar_x
"""

from __future__ import absolute_import
import ctypes
import logging
import numpy as np
from sba.crsm import Crsm
from sba.projections import FuncProjection, FjacProjJacobian, FuncProjectionSimple, FjacProjJacobianSimple
from sba.options import OPTS_SIZE
from sba.info import INFO_SIZE
import sba.options
import sys
import os


if 'linux' in sys.platform:
    # This line actually loads the sba shared object
    _libsba = ctypes.CDLL(os.path.abspath("libsba.so"))
    # On Ubuntu 12.04 this is /usr/local/lib/libsba.so, which is a symbolic link
    # to /usr/local/lib/libsba.so.1.6.4 currently

elif sys.platform == 'darwin':
    # On Mac you may wish to use .dylib extension instead; be sure to rename
    # the .so in the Makefile or when installing and use the correct Mac 
    # locations here instead.
    _libsba = ctypes.CDLL("libsba.dylib")
    # THIS MAY NEED DEBUGGING

elif sys.platform == 'win32' or sys.platform == 'win64':
    # on Windows,
    try:
        _libsba = ctypes.CDLL("libsba.dll")
    except:
        _libsba = ctypes.CDLL("libsba_p3.dll")
    # THIS MAY NEED DEBUGGING

# "expert" drivers from sba.h (BU modified version 1.6.1)
# function prototype for sba_motstr_levmar_x
_libsba.sba_motstr_levmar_x.argtypes = [ctypes.c_int, # n num of points
                                        ctypes.c_int, # ncon number not to be modified
                                        ctypes.c_int, # m num of images
                                        ctypes.c_int, # mcon number not to be modified
                                        ctypes.c_char_p, # visibility mask
                                        ctypes.POINTER(ctypes.c_double), # initial parameter vector
                                        ctypes.c_int, # cnp number of params for ONE camera
                                        ctypes.c_int, # pnp number of params for ONE point eg 3
                                        ctypes.POINTER(ctypes.c_double), # x measurements vector
                                        ctypes.POINTER(ctypes.c_double), # covx measurements covariance
                                        ctypes.c_int, # mnp number of params for EACH measurement eg 2
                                        FuncProjection, # functional relation describing measurements?
                                        FjacProjJacobian, # evaluate sparse Jacobian dX/dp
                                        ctypes.c_void_p, # adata, additional data to func, fjac
                                        ctypes.c_int, # itmax max iterations
                                        ctypes.c_int, # verbosity
                                        ctypes.c_double * OPTS_SIZE, # opts options
                                        ctypes.c_double * INFO_SIZE] # info 
_libsba.sba_motstr_levmar_x.restype = ctypes.c_int 
# Python handle
_MotionStructureExpert = _libsba.sba_motstr_levmar_x
_MotionStructureExpert.__doc__ = """
function prototype for sba routine from libsba
long int sba_motstr_levmar_x(long int n, long int ncon, long int m, long int mcon,
char *vmask, double *p, long int cnp, long int pnp, double *x, double *covx,
long int mnp, (func - a projection function), (fjac - a Jacobian function), 
void *adata, long int itmax, long int verbose, double opts[OPTS_SIZE], 
double info[INFO_SIZE]

This does sba adjustment with motion and structure in expert mode. 
"""
_MotionStructureExpert.__str__ = "MotionStructureExpert"


# function prototype for sba_mot_levmar_x
_libsba.sba_mot_levmar_x.argtypes = [ctypes.c_int, # n num of points
                                     ctypes.c_int, # m num of images
                                     ctypes.c_int, # mcon number not to be modified
                                     ctypes.c_char_p, # visibility mask
                                     ctypes.POINTER(ctypes.c_double), # initial parameter vector
                                     ctypes.c_int, # cnp number of params for ONE camera
                                     #ctypes.c_int, # pnp number of params for ONE point eg 3
                                     ctypes.POINTER(ctypes.c_double), # x measurements vector
                                     ctypes.POINTER(ctypes.c_double), # covx measurements covariance
                                     ctypes.c_int, # mnp number of params for EACH measurement eg 2
                                     FuncProjection, # functional relation describing measurements?
                                     FjacProjJacobian, # evaluate sparse Jacobian dX/dp
                                     ctypes.c_void_p, # adata, additional data to func, fjac
                                     ctypes.c_int, # itmax max iterations
                                     ctypes.c_int, # verbosity
                                     ctypes.c_double * OPTS_SIZE, # opts options
                                     ctypes.c_double * INFO_SIZE] # info 
_libsba.sba_mot_levmar_x.restype = ctypes.c_int 
# Python handle
_MotionExpert = _libsba.sba_mot_levmar_x
_MotionExpert.__doc__ = """
function prototype for sba routine from libsba
long int sba_mot_levmar_x(long int n, long int m, long int mcon,
char *vmask, double *p, long int cnp, long int pnp, double *x, double *covx,
long int mnp, (func - a projection function), (fjac - a Jacobian function), 
void *adata, long int itmax, long int verbose, double opts[OPTS_SIZE], 
double info[INFO_SIZE]

This does sba adjustment with motion in expert mode. 
"""
_MotionExpert.__str__ = "MotionExpert"

# function prototype for sba_str_levmar_x
_libsba.sba_str_levmar_x.argtypes = [ctypes.c_int, # n num of points
                                     ctypes.c_int, # ncon number not to be modified
                                     ctypes.c_int, # m num of images
                                     ctypes.c_char_p, # visibility mask
                                     ctypes.POINTER(ctypes.c_double), # initial parameter vector
                                     #ctypes.c_int, # cnp number of params for ONE camera
                                     ctypes.c_int, # pnp number of params for ONE point eg 3
                                     ctypes.POINTER(ctypes.c_double), # x measurements vector
                                     ctypes.POINTER(ctypes.c_double), # covx measurements covariance
                                     ctypes.c_int, # mnp number of params for EACH measurement eg 2
                                     FuncProjection, # functional relation describing measurements?
                                     FjacProjJacobian, # evaluate sparse Jacobian dX/dp
                                     ctypes.c_void_p, # adata, additional data to func, fjac
                                     ctypes.c_int, # itmax max iterations
                                     ctypes.c_int, # verbosity
                                     ctypes.c_double * OPTS_SIZE, # opts options
                                     ctypes.c_double * INFO_SIZE] # info 
_libsba.sba_str_levmar_x.restype = ctypes.c_int 
# Python handle
_StructureExpert = _libsba.sba_str_levmar_x
_StructureExpert.__doc__ = """
function prototype for sba routine from libsba
long int sba_motstr_levmar_x(long int n, long int ncon, long int m
char *vmask, double *p, long int cnp, long int pnp, double *x, double *covx,
long int mnp, (func - a projection function), (fjac - a Jacobian function), 
void *adata, long int itmax, long int verbose, double opts[OPTS_SIZE], 
double info[INFO_SIZE]

This does sba adjustment with structure in expert mode. 
"""
_StructureExpert.__str__ = "StructureExpert"




# Simple drivers
_libsba.sba_motstr_levmar.argtypes = [ctypes.c_int, # n
                                      ctypes.c_int, # ncon
                                      ctypes.c_int, # m
                                      ctypes.c_int, # mcon
                                      ctypes.c_char_p, # *vmask
                                      ctypes.POINTER(ctypes.c_double), # *p
                                      ctypes.c_int, # cnp
                                      ctypes.c_int, # pnp
                                      ctypes.POINTER(ctypes.c_double), # *x
                                      ctypes.POINTER(ctypes.c_double), # *covx
                                      ctypes.c_int, # mnp
                                      FuncProjectionSimple,
                                      FjacProjJacobianSimple,
                                      ctypes.c_void_p, # *adata
                                      ctypes.c_int, # itmax
                                      ctypes.c_int, # verbose
                                      ctypes.c_double * OPTS_SIZE, # double *opts
                                      ctypes.c_double * INFO_SIZE] # double *info
_libsba.sba_motstr_levmar.restype = ctypes.c_int
_MotionStructureSimple = _libsba.sba_motstr_levmar
_MotionStructureSimple.__doc__ = """
function prototype for sba routine from libsba
long int sba_motstr_levmar(long int n, long int ncon, long int m, long int mcon,
char *vmask, double *p, long int cnp, long int pnp, double *x, double *covx,
long int mnp, (func - a projection function), (fjac - a Jacobian function), 
void *adata, long int itmax, long int verbose, double opts[OPTS_SIZE], 
double info[INFO_SIZE]

This does sba adjustment with motion and structure in simple mode. 
"""
_MotionStructureSimple.__str__ = "MotionStructureSimple"

_libsba.sba_mot_levmar.argtypes = [ctypes.c_int, # n
                                   ctypes.c_int, # m
                                   ctypes.c_int, # mcon
                                   ctypes.c_char_p, # *vmask
                                   ctypes.POINTER(ctypes.c_double), # *p
                                   ctypes.c_int, # cnp
                                   ctypes.c_int, # pnp
                                   ctypes.POINTER(ctypes.c_double), # *x
                                   ctypes.POINTER(ctypes.c_double), # *covx
                                   ctypes.c_int, # mnp
                                   FuncProjectionSimple,
                                   FjacProjJacobianSimple,
                                   ctypes.c_void_p, # *adata
                                   ctypes.c_int, # itmax
                                   ctypes.c_int, # verbose
                                   ctypes.c_double * OPTS_SIZE, # double *opts
                                   ctypes.c_double * INFO_SIZE] # double *info
_libsba.sba_mot_levmar.restype = ctypes.c_int
_MotionSimple = _libsba.sba_mot_levmar
_MotionSimple.__doc__ = """
function prototype for sba routine from libsba
long int sba_mot_levmar(long int n, long int m, long int mcon,
char *vmask, double *p, long int cnp, long int pnp, double *x, double *covx,
long int mnp, (func - a projection function), (fjac - a Jacobian function), 
void *adata, long int itmax, long int verbose, double opts[OPTS_SIZE], 
double info[INFO_SIZE]

This does sba adjustment with motion and structure in simple mode. 
"""
_MotionSimple.__str__ = "MotionSimple"

_libsba.sba_str_levmar.argtypes = [ctypes.c_int, # n
                                   ctypes.c_int, # ncon
                                   ctypes.c_int, # m
                                   ctypes.c_char_p, # *vmask
                                   ctypes.POINTER(ctypes.c_double), # *p
                                   ctypes.c_int, # cnp
                                   ctypes.c_int, # pnp
                                   ctypes.POINTER(ctypes.c_double), # *x
                                   ctypes.POINTER(ctypes.c_double), # *covx
                                   ctypes.c_int, # mnp
                                   FuncProjectionSimple,
                                   FjacProjJacobianSimple,
                                   ctypes.c_void_p, # *adata
                                   ctypes.c_int, # itmax
                                   ctypes.c_int, # verbose
                                   ctypes.c_double * OPTS_SIZE, # double *opts
                                   ctypes.c_double * INFO_SIZE] # double *info
_libsba.sba_str_levmar.restype = ctypes.c_int
_StructureSimple = _libsba.sba_str_levmar
_StructureSimple.__doc__ = """
function prototype for sba routine from libsba
long int sba_motstr_levmar(long int n, long int ncon, long int m,
char *vmask, double *p, long int cnp, long int pnp, double *x, double *covx,
long int mnp, (func - a projection function), (fjac - a Jacobian function), 
void *adata, long int itmax, long int verbose, double opts[OPTS_SIZE], 
double info[INFO_SIZE]

This does sba adjustment with motion and structure in simple mode. 
"""
_StructureSimple.__str__ = "StructureSimple"



"""
Not implemented:
CRSM routines 
Linear algebra helper routines
"""

def whichRoutine(options):
    """
    Dispatcher for which sba routine to use based on options
    
    input:
    options - Options object specifying sba options

    returns:
    a Python pointer to a C function (ctypes.funcptr) for sba projection.
    e.g. sba_levmar_motstr_x, sba_levmar_str_x, etc. 

    Raises SbaError if options are not recognized
    """
    #logging.debug("routines.whichRoutine() called with {0}".format(options))
    if options.expert:
        if options.motstruct==sba.options.OPTS_MOTSTRUCT:
            return _MotionStructureExpert
        elif options.motstruct==sba.options.OPTS_MOT:
            return _MotionExpert
        elif options.motstruct==sba.options.OPTS_STRUCT:
            return _StructureExpert
        else:
            raise SbaError("Unrecognized sba option")
    else:
        if options.motstruct==sba.options.OPTS_MOTSTRUCT:
            return _MotionStructureSimple
        elif options.motstruct==sba.options.OPTS_MOT:
            return _MotionSimple
        elif options.motstruct==sba.options.OPTS_STRUCT:
            return _StructureSimple
        else:
            raise SbaError("Unrecognized sba option")



#!/usr/bin/env python
"""
options.py
Dennis Evangelista, 2013

When C calling routines ask for it, Options is used to help figure out:

1. Which projection and jacobian to call?
       use magic decoder function in sba.projections

2. What options array to pass
       use Options.optsArray() , which calls Options.opts.toC()

3. What maxiter, verbose to pass, converted to C...

4. What Globs to pass, converted to a C structure
"""

from __future__ import absolute_import
from __future__ import print_function
import ctypes
import numpy as np
import logging







# These are stopping criteria for Levenberg-Marquardt algorithm
OPTS_SIZE = 5 # Size of LM options array of double
_INIT_MU = 1.0E-3 # mu, LM parameter
_STOP_THRESH = 1.0E-12 # stopping threshold
_MAXITER = 1000 # max iterations
_MAXITER2 = 1500 # unused? 

COptsType = ctypes.c_double*OPTS_SIZE

class LMOptions(object):
    """
    Python friendly implementation of sba Levenberg-Marquardt options

    attributes:
    mu - default 1e-3, LM parameter
    J^Teps_inf - default 1e-12, LM parameter
    DP - default 1e-12, LM parameter
    eps - default 1e-12, LM parameter
    eps/stuff - default 0.0, LM parameter
    """
    def __init__(self):
        self.__mu = _INIT_MU # LM parameter
        self.__JTeps = _STOP_THRESH # LM parameter
        self.__DP = _STOP_THRESH # LM parameter
        self.__eps = _STOP_THRESH # LM parameter
        self.__epsrr = 0.0 # LM parameter
        self.__optsarray = np.array([self.__mu, self.__JTeps, self.__DP,
                                     self.__eps, self.__epsrr],dtype=np.double)
        
        super(LMOptions, self).__init__()

    def _getmu(self):
        return self.__mu
    def _setmu(self, value):
        self.__mu = value
        self.__optsarray[0] = value
    mu = property(_getmu, _setmu,doc="mu parameter for Levenberg-Marquardt")

    def _getJTeps(self):
        return self.__JTeps
    def _setJTeps(self, value):
        self.__JTeps = value
        self.__optsarray[1] = value
    JTeps = property(_getJTeps, _setJTeps,doc="J^Teps_inf parameter for LM")

    def _getDP(self):
        return self.__DP
    def _setDP(self, value):
        self.__DP = value
        self.__optsarray[2] = value
    DP = property(_getDP, _setDP,doc="DP parameter for Levenberg-Marquardt")

    def _geteps(self):
        return self.__eps
    def _seteps(self, value):
        self.__eps = value
        self.__optsarray[3] = value
    eps = property(_geteps, _seteps,doc="epsilon stopping param for LM")

    def _getepsrr(self):
        return self.__epsrr
    def _setepsrr(self, value):
        self.__epsrr = value
        self.__optsarray[4] = value
    epsrr = property(_getepsrr, _setepsrr,doc="eps/stuff stopping param for LM")

    def _getoptsarray(self):
        return self.__optsarray
    def _setoptsarray(self,newoptsarray):
        self.__optsarray = np.array(newoptsarray,dtype=np.double)
        self.__mu = self.__optsarray[0]
        self.__JTeps = self.__optsarray[1]
        self.__DP = self.__optsarray[2]
        self.__eps = self.__optsarray[3]
        self.__epsrr = self.__optsarray[4]
    optsarray = property(_getoptsarray, _setoptsarray,doc="5 element options array for LM")

    def toC(self):
        """Returns a C-passable (double ``*``) pointer to the options array"""
        return COptsType(self.__optsarray[0],self.__optsarray[1],
                         self.__optsarray[2],self.__optsarray[3],
                         self.__optsarray[4])


    









class Globs(object):
    """
    Globs for passing additional options to projection and Jacobian 
    functions in sba.

    attributes:
    rot0params, default None, specifies initial rotation parameters?
    intrcalib, default None, specifies intrinsics if all cams held identical
    nccalib, default OPTS_FIX4_INTR, how many intrinsics to hold fixed
    ncdist, default OPTS_FIX3_DIST, how many distortion coeffs to hold fixed
    ptparams, default None, used when adjusting for cam params only
    camparams, default None, used when adjusting for structure only

    dimensions:
    cnp, default 3+3+5+5, number of parameters for each camera
    pnp, default 2, number of parameters for each image point (uv)
    mnp, default 3, number of parameters for each world point (xyz)
    """
    def __init__(self):
        """Create Globs object with default parameters"""
        self.__rot0params = None # initial rotation parameters 
        self.intrcalib = None # calibration when all cams fixed and identical
        self.nccalib = OPTS_FIX4_INTR # hold this many intrinsics fixed
        self.ncdist = OPTS_FIX3_DIST # hold this many dist coeffs fixed
        self.ptparams = None # needed when adjusting for cam params only
        self.camparams = None # needed when adjusting for structure only

        self._cnp = 5+5+3+3 # must set this, number of parameters per camera
        self.pnp = 2 # numbers for an image point
        self.mnp = 3 # numbers for a world point

        super(Globs,self).__init__()

    def _getrot0params(self):
        return self.__rot0params
    def _setrot0params(self,value):
        if value is None:
            self.__rot0params = None
            self.__Crot0params = None
        else:
            self.__rot0params = np.array(value,dtype=np.double)
            self.__Crot0params = self.__rot0params.ravel()
    rot0params = property(_getrot0params,_setrot0params,doc="initial rotation parameters")

    @property
    def Crot0params(self):
        return self.__Crot0params.ctypes.data_as(ctypes.POINTER(ctypes.c_double))
        
    def _getcnp(self):
        return self._cnp
    def _setcnp(self,val):
        self._cnp = val
        if self._cnp != 16:
            self.ncdist = -9999; 
    cnp = property(_getcnp,_setcnp,doc="Number of parameters per camera")
    
    @property 
    def hasFixedCal(self):
        return self.nccalib == OPTS_FIX5_INTR 

    @property
    def hasDistortion(self):
        return self.cnp == 5+5+3+3

    def toC(self):
        print(self.ncdist)
        """Returns Globs as a C Structure globs``_`` for passing to sba C routines"""
        if type(self.intrcalib) is np.ndarray:
            Cintrcalib = self.intrcalib.ctypes.data_as(ctypes.POINTER(ctypes.c_double))
        else:
            Cintrcalib = None
        if type(self.ptparams) is np.ndarray:
            Cptparams = self.ptparams.ctypes.data_as(ctypes.POINTER(ctypes.c_double))
        else:
            Cptparams = None
        if type(self.camparams) is np.ndarray:
            Ccamparams = self.camparams.ctypes.data_as(ctypes.POINTER(ctypes.c_double))
        else:
            Ccamparams = None
        return ctypes.pointer(_Globs_(self.Crot0params, # * double
                                      Cintrcalib, # * double
                                      ctypes.c_int(self.nccalib), # int
                                      ctypes.c_int(self.ncdist),  # int
                                      ctypes.c_int(self.cnp), # int
                                      ctypes.c_int(self.pnp), # int
                                      ctypes.c_int(self.mnp), # int
                                      Cptparams,  # * double
                                      Ccamparams)) # * double





class _Globs_(ctypes.Structure):
    """ctypes Structure for passing Globs to sba only"""
    _fields_ = [("rot0params", ctypes.POINTER(ctypes.c_double)),
                ("intrcalib",ctypes.POINTER(ctypes.c_double)),
                ("nccalib",ctypes.c_int),
                ("ncdist",ctypes.c_int),
                ("cnp",ctypes.c_int),
                ("pnp",ctypes.c_int),
                ("mnp",ctypes.c_int),
                ("ptparams",ctypes.POINTER(ctypes.c_double)),
                ("camparams",ctypes.POINTER(ctypes.c_double))]

        














# SBA options

# 1. Use expert mode (default) or simple?
# 2. Allow sba to alter camera?
OPTS_CAMS = 0 # camera intrinsics and distortion (default)
OPTS_CAMS_NODIST = 1 # camera intrinsics without distortion
OPTS_NO_CAMS = 2 # don't optimize cameras

# 3. Allow sba to alter motion, structure, or both
OPTS_MOTSTRUCT = 0 # motion and structure, default
OPTS_MOT = 1 # just motion
OPTS_STRUCT = 2 # just structure

# 4. Use analytic Jacobian (default) or not? 

# 5. Verbose or not? 
# What to print? 
OPTS_PRINT_NONE = -1
OPTS_PRINT_MOTSTRUCT = 0
OPTS_PRINT_MOT = 1
OPTS_PRINT_STRUCT = 2

# 6. Which camera intrinsics to hold fixed?
OPTS_FIX0_INTR = 0 # all free 
OPTS_FIX1_INTR = 1 # skew is fixed
OPTS_FIX2_INTR = 2 # skew and aspect ratio are fixed
OPTS_FIX3_INTR = 3 # not meaningful
OPTS_FIX4_INTR = 4 # skew, AR, and principal point fixed
OPTS_FIX5_INTR = 5 # all fixed
OPTS_FREE_INTR = OPTS_FIX0_INTR # synonym for all free
OPTS_FIX_SKEW = OPTS_FIX1_INTR # synonym 
OPTS_FIX_SKEW_AR = OPTS_FIX2_INTR # synonym
OPTS_FIX_SKEW_AR_PP = OPTS_FIX4_INTR # synonym
OPTS_FIXALL_INTR = OPTS_FIX5_INTR # synonym

# 7. Which distortion coeffs to hold fixed? 
OPTS_FIX0_DIST = 0 # all free
OPTS_FIX1_DIST = 1 # R6 fixed
OPTS_FIX2_DIST = 2 # R6, T2 fixed
OPTS_FIX3_DIST = 3 # R6, T2, T1 fixed
OPTS_FIX4_DIST = 4 # R6, T2, T1, R4 fixed
OPTS_FIX5_DIST = 5 # all fixed
OPTS_FREE_DIST = OPTS_FIX0_DIST # synonym for all free
OPTS_FIX_R6 = OPTS_FIX1_DIST # synonym
OPTS_FIX_TANG = OPTS_FIX3_DIST # synonym
OPTS_FIXALL_DIST = OPTS_FIX5_DIST #synonym

class Options(LMOptions,Globs):
    """
    Sparse bundle adjustment options settings

    attributes:
    expert - Use expert mode (default True)
    camera - Alter intrinsics and distortion? (default OPTS_CAMS = both)
    motstruct - Do sba for motions and structure? (default OPTS_MOTSTRUCT = both)
    analjac - Use analytic Jacobian (default True)
    verbose - (default True)
    printwhat - (default OPTS_PRINT_MOTSTRUCT = both)
    maxiter - Max iterations (default 100)

    Also a subclass of LMOptions, which has the Levenberg-Marquardt options
    and Globs, which has additional options passed to the projection and
    Jacobian functions. See those for additional options provided:

    attributes inherited from Globs:
    rot0params, default None, specifies initial rotation parameters?
    intrcalib, default None, specifies intrinsics if all cams held identical
    nccalib, default OPTS_FIX4_INTR, how many intrinsics to hold fixed
    ncdist, default OPTS_FIX3_DIST, how many distortion coeffs to hold fixed
    ptparams, default None, used when adjusting for cam params only
    camparams, default None, used when adjusting for structure only
    cnp, default 3+3+5+5, number of parameters for each camera
    pnp, default 2, number of parameters for each image point (uv)
    mnp, default 3, number of parameters for each world point (xyz)
 
    attributes inherited from LMOptions:
    mu - default 1e-3, LM parameter
    J^Teps_inf - default 1e-12, LM parameter
    DP - default 1e-12, LM parameter
    eps - default 1e-12, LM parameter
    eps/stuff - default 0.0, LM parameter

    pro tip: the base C routines seem to get confused if you call 
    OPTS_CAMS_NODIST plus OPTS_FIX5_INTR or OPTS_FIX5_DIST... (for example)
    For our work at UNC we usually supply 5+5+3+3 camera parameters,
    if you're doing that, use OPTS_CAMS (the cameras do have dist coeffs),
    so that it knows how to properly interpret what you want fixed. 
    """
    def __init__(self): 
        self.expert = True # Use expert mode
        self.camera = OPTS_CAMS # sba can alter camera intrinsics+dist
        self.motstruct = OPTS_MOTSTRUCT # sba for motion and structure
        self.analjac = True # use analytic jacobian
        self.verbose = True # verbose output
        self.printwhat = OPTS_PRINT_MOTSTRUCT
        self.maxiter = _MAXITER2 # max iterations 100
        super(Options, self).__init__()
        #self.opts = LMOptions() # numerical options for Levenberg-Marquardt
        #self.globs = Globs()

    def maxIterToC(self):
        return ctypes.c_int(self.maxiter)
    def verboseToC(self):
        if self.verbose:
            return ctypes.c_int(1)
        else:
            return ctypes.c_int(0) 
    def globsToC(self):
        return Globs.toC(self) # Why can't I do this using super? 
    def optsToC(self):
        return LMOptions.toC(self) # Why can't I do this using super? 
    def toC(self):
        return (self.maxIterToC(),self.verboseToC(),self.globsToC(),
                self.optsToC())

    def __str__(self):
        """Human readable representation of sba.Options()"""
        return "<sba.Options() object, expert {0}, motstruct {1}>".format(self.expert,self.motstruct)

    def _fromInput(cls,cameras,points):
        """
        Alternate constructor (a class method) to 
        create default Options object with values compatible
        with input objects cameras and points
        """
        newoptions = cls()
        newoptions.ncameras = cameras.ncameras
        if cameras.cnp==5+5+3+3:
            newoptions.camera = OPTS_CAMS
        elif cameras.cnp==5+3+3:
            newoptions.camera = OPTS_CAMS_NODIST
        elif cameras.cnp==3+3:
            newoptions.camera = OPTS_NOCAMS
        newoptions.cnp = cameras.cnp
        newoptions.rot0params = cameras.rot0params
        newoptions.mnp = points.mnp
        newoptions.pnp = points.pnp
        return newoptions
    fromInput = classmethod(_fromInput)

    # pretty print method? 















#!/usr/bin/env python
"""
errors.py
Dennis Evangelista 2013

For handling errors in wrapped sba routines
"""


class SbaError(Exception):
    """Base class for exceptions in python-sba code"""
    def __init__(self,value):
        """Value should be the cause of the SbaError"""
        self.value = value
        #super(Exception,self).__init__()
    def __str__(self):
        """Returns string representation of SbaError"""
        return "{0}".format(self.value)



#!/usr/bin/env python
"""
projections.py
Python wrapper for projections and jacobians from eucsbademo by Lourakis
Wrapper by Dennis Evangelista, 2013
"""

from __future__ import absolute_import
import ctypes
import logging
import numpy as np
import sba.options
from sba.crsm import Crsm
#from sba.options import Globs_
from sba.errors import SbaError
import sys
import os


if 'linux' in sys.platform:
    # This line actually loads the sba shared object from /usr/local/lib
    _libsbaprojs = ctypes.CDLL(os.path.abspath("libsbaprojs.so"))
    #print(os.path.abspath('libsbaprojs.so'))
    # On Ubuntu 12.04, this should be in /usr/local/lib/libsbaprojs.so as a link
    # to /usr/local/lib/libsbaprojs.so.1.6.4

elif sys.platform == 'darwin':
    # On Mac, you may wish to use .dylib rather than .so and set the paths
    # accordingly. 
    _libsbaprojs = ctypes.CDLL("libsbaprojs.dylib")
    # THIS MAY NEED DEBUGGING

elif sys.platform == 'win32' or sys.platform == 'win64':
    # On Windows,
    try:
        _libsbaprojs = ctypes.CDLL("libsbaprojs.dll")
    except:
        _libsbaprojs = ctypes.CDLL("libsbaprojs_p3.dll")
    # THIS MAY NEED DEBUGGING


# This is used for going from Python into C
FuncProjection = ctypes.CFUNCTYPE(None, # return type void
                                  ctypes.POINTER(ctypes.c_double), #p
                                  ctypes.POINTER(Crsm), #idxij
                                  ctypes.POINTER(ctypes.c_int), #rcidxs
                                  ctypes.POINTER(ctypes.c_int), #rcsubs
                                  ctypes.POINTER(ctypes.c_double), #hx
                                  ctypes.c_void_p) #adata
FuncProjection.__doc__ = """
Function prototype for callable projection func in expert mode sba:
void (*func) (double *p, struct sba_crsm *idxij, int *rcidxs, 
int *rcsubs, double *hx, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""

# This is used for going from Python into C
FjacProjJacobian = ctypes.CFUNCTYPE(None, # return type void
                                    ctypes.POINTER(ctypes.c_double), #p
                                    ctypes.POINTER(Crsm), #idxij
                                    ctypes.POINTER(ctypes.c_int), #rcidxs
                                    ctypes.POINTER(ctypes.c_int), #rcsubs
                                    ctypes.POINTER(ctypes.c_double), #jac
                                    ctypes.c_void_p) #adata
FjacProjJacobian.__doc__ = """
Function prototype for callable Jacobian fjac in expert mode sba:
void (*fjac) (double *p, struct sba_crsm *idxij, int *rcidxs, 
int *rcsubs, double *hx, void *adata)

Use to wrap a Python function when sending a Jacobian function from 
Python into C.
"""

FuncProjectionSimple = ctypes.CFUNCTYPE(None, # return type void
                                        ctypes.c_int, # j
                                        ctypes.c_int, # i
                                        ctypes.POINTER(ctypes.c_double), # *bi
                                        ctypes.POINTER(ctypes.c_double), # *xij
                                        ctypes.c_void_p) # adata
FuncProjectionSimple.__doc__ = """
Function prototype for callable projection func in simple mode sba:
void (*proj) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""

FjacProjJacobianSimple = ctypes.CFUNCTYPE(None, # return type void
                                          ctypes.c_int, # j
                                          ctypes.c_int, # i
                                          ctypes.POINTER(ctypes.c_double), # *bi
                                          ctypes.POINTER(ctypes.c_double), # *xij
                                          ctypes.c_void_p) # adata
FuncProjectionSimple.__doc__ = """
Function prototype for callable Jacobian fjac in simple mode sba:
void (*projac) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a Jacobian function from 
Python into C.
"""






# projection functions and their Jacobians
# full sparse bundle adjustment
_libsbaprojs.img_projsRTS_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                        ctypes.POINTER(Crsm),
                                        ctypes.POINTER(ctypes.c_int),
                                        ctypes.POINTER(ctypes.c_int),
                                        ctypes.POINTER(ctypes.c_double),
                                        ctypes.c_void_p]
_libsbaprojs.img_projsRTS_x.restype = None
_libsbaprojs.img_projsRTS_x.__str__ = "MotionsStructureExpert_NoCameras"
# Python handle
_MotionStructureExpert_NoCameras = _libsbaprojs.img_projsRTS_x
_MotionStructureExpert_NoCameras.__doc__="""
Function prototype for callable projection func in expert mode sba:
void (*func) (double *p, struct sba_crsm *idxij, int *rcidxs, 
int *rcsubs, double *hx, void *adata)

Does projection from P to X. 
"""

_libsbaprojs.img_projsRTS_jac_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                            ctypes.POINTER(Crsm),
                                            ctypes.POINTER(ctypes.c_int),
                                            ctypes.POINTER(ctypes.c_int),
                                            ctypes.POINTER(ctypes.c_double),
                                            ctypes.c_void_p]
_libsbaprojs.img_projsRTS_jac_x.restype = None
_libsbaprojs.img_projsRTS_jac_x.__str__ = "MotionsStructureJacobianExpert_NoCameras"
# Python handle
_MotionStructureJacobianExpert_NoCameras = _libsbaprojs.img_projsRTS_jac_x
_MotionStructureJacobianExpert_NoCameras.__doc__="""
Function prototype for callable Jacobian fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Does projection from P to X. 
"""

# sparse bundle adjustment with camera parameters only
_libsbaprojs.img_projsRT_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                       ctypes.POINTER(Crsm),
                                       ctypes.POINTER(ctypes.c_int),
                                       ctypes.POINTER(ctypes.c_int),
                                       ctypes.POINTER(ctypes.c_double),
                                       ctypes.c_void_p]
_libsbaprojs.img_projsRT_x.restype = None
_libsbaprojs.img_projsRT_x.__str__ = "MotionExpert_NoCameras"
# Python handle
_MotionExpert_NoCameras = _libsbaprojs.img_projsRT_x
_MotionExpert_NoCameras.__doc__="""
Function prototype for callable projection func in expert mode sba:
void (*func) (double *p, struct sba_crsm *idxij, int *rcidxs, 
int *rcsubs, double *hx, void *adata)

Does projection from P to X. 
"""

_libsbaprojs.img_projsRT_jac_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                           ctypes.POINTER(Crsm),
                                           ctypes.POINTER(ctypes.c_int),
                                           ctypes.POINTER(ctypes.c_int),
                                           ctypes.POINTER(ctypes.c_double),
                                           ctypes.c_void_p]
_libsbaprojs.img_projsRT_jac_x.restype = None
_libsbaprojs.img_projsRT_jac_x.__str__ = "MotionJacobianExpert_NoCameras"
# Python handle
_MotionJacobianExpert_NoCameras = _libsbaprojs.img_projsRT_jac_x
_MotionJacobianExpert_NoCameras.__doc__="""
Function prototype for callable Jacobian fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Does projection from P to X. 
"""

# sparse bundle adjustment with structure parameters only
_libsbaprojs.img_projsS_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                      ctypes.POINTER(Crsm),
                                      ctypes.POINTER(ctypes.c_int),
                                      ctypes.POINTER(ctypes.c_int),
                                      ctypes.POINTER(ctypes.c_double),
                                      ctypes.c_void_p]
_libsbaprojs.img_projsS_x.restype = None
_libsbaprojs.img_projsS_x.__str__ = "StructureExpert_NoCameras"
# Python handle
_StructureExpert_NoCameras = _libsbaprojs.img_projsS_x
_StructureExpert_NoCameras.__doc__="""
Function prototype for callable projection func in expert mode sba:
void (*func) (double *p, struct sba_crsm *idxij, int *rcidxs, 
int *rcsubs, double *hx, void *adata)

Does projection from P to X. 
"""

_libsbaprojs.img_projsS_jac_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                          ctypes.POINTER(Crsm),
                                          ctypes.POINTER(ctypes.c_int),
                                          ctypes.POINTER(ctypes.c_int),
                                          ctypes.POINTER(ctypes.c_double),
                                          ctypes.c_void_p]
_libsbaprojs.img_projsS_jac_x.restype = None
_libsbaprojs.img_projsS_jac_x.__str__ = "StructureJacobianExpert_NoCameras"
# Python handle
_StructureJacobianExpert_NoCameras = _libsbaprojs.img_projsS_jac_x
_StructureJacobianExpert_NoCameras.__doc__="""
Function prototype for callable Jacobian fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Does projection from P to X. 
"""

# Cameras with distortion
_libsbaprojs.img_projsKDRTS_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                          ctypes.POINTER(Crsm),
                                          ctypes.POINTER(ctypes.c_int),
                                          ctypes.POINTER(ctypes.c_int),
                                          ctypes.POINTER(ctypes.c_double),
                                          ctypes.c_void_p]
_libsbaprojs.img_projsKDRTS_x.restype = None
_libsbaprojs.img_projsKDRTS_x.__str__ = "MotionsStructureExpert_Cameras"
# Python handle
_MotionStructureExpert_Cameras = _libsbaprojs.img_projsKDRTS_x
_MotionStructureExpert_Cameras.__doc__="""
Function prototype for callable projection func in expert mode sba:
void (*func) (double *p, struct sba_crsm *idxij, int *rcidxs, 
int *rcsubs, double *hx, void *adata)

Does projection from P to X. 
"""

_libsbaprojs.img_projsKDRTS_jac_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                              ctypes.POINTER(Crsm),
                                              ctypes.POINTER(ctypes.c_int),
                                              ctypes.POINTER(ctypes.c_int),
                                              ctypes.POINTER(ctypes.c_double),
                                              ctypes.c_void_p]
_libsbaprojs.img_projsKDRTS_jac_x.restype = None
_libsbaprojs.img_projsKDRTS_jac_x.__str__ = "MotionsStructureJacobianExpert_Cameras"
# Python handle
_MotionStructureJacobianExpert_Cameras = _libsbaprojs.img_projsKDRTS_jac_x
_MotionStructureJacobianExpert_Cameras.__doc__="""
Function prototype for callable projection fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Does projection from P to X. 
"""

_libsbaprojs.img_projsKDRT_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                         ctypes.POINTER(Crsm),
                                         ctypes.POINTER(ctypes.c_int),
                                         ctypes.POINTER(ctypes.c_int),
                                         ctypes.POINTER(ctypes.c_double),
                                         ctypes.c_void_p]
_libsbaprojs.img_projsKDRT_x.restype = None
_libsbaprojs.img_projsKDRT_x.__str__ = "MotionExpert_Cameras"
# Python handle
_MotionExpert_Cameras = _libsbaprojs.img_projsKDRT_x
_MotionExpert_Cameras.__doc__="""
Function prototype for callable projection func in expert mode sba:
void (*func) (double *p, struct sba_crsm *idxij, int *rcidxs, 
int *rcsubs, double *hx, void *adata)

Does projection from P to X. 
"""

_libsbaprojs.img_projsKDRT_jac_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                             ctypes.POINTER(Crsm),
                                             ctypes.POINTER(ctypes.c_int),
                                             ctypes.POINTER(ctypes.c_int),
                                             ctypes.POINTER(ctypes.c_double),
                                             ctypes.c_void_p]
_libsbaprojs.img_projsKDRT_jac_x.restype = None
_libsbaprojs.img_projsKDRT_jac_x.__str__ = "MotionJacobianExpert_Cameras"
# Python handle
_MotionJacobianExpert_Cameras = _libsbaprojs.img_projsKDRT_jac_x
_MotionJacobianExpert_Cameras.__doc__="""
Function prototype for callable Jacobian fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Does projection from P to X. 
"""

_libsbaprojs.img_projsKDS_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                        ctypes.POINTER(Crsm),
                                        ctypes.POINTER(ctypes.c_int),
                                        ctypes.POINTER(ctypes.c_int),
                                        ctypes.POINTER(ctypes.c_double),
                                        ctypes.c_void_p]
_libsbaprojs.img_projsKDS_x.restype = None
_libsbaprojs.img_projsKDS_x.__str__ = "StructureExpert_Cameras"
# Python handle
_StructureExpert_NoCameras = _libsbaprojs.img_projsKDS_x
_StructureExpert_NoCameras.__doc__="""
Function prototype for callable projection func in expert mode sba:
void (*func) (double *p, struct sba_crsm *idxij, int *rcidxs, 
int *rcsubs, double *hx, void *adata)

Does projection from P to X. 
"""

_libsbaprojs.img_projsKDS_jac_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                            ctypes.POINTER(Crsm),
                                            ctypes.POINTER(ctypes.c_int),
                                            ctypes.POINTER(ctypes.c_int),
                                            ctypes.POINTER(ctypes.c_double),
                                            ctypes.c_void_p]
_libsbaprojs.img_projsKDS_jac_x.restype = None
_libsbaprojs.img_projsKDS_jac_x.__str__ = "StructureJacobianExpert_Cameras"
# Python handle
_StructureJacobianExpert_Cameras = _libsbaprojs.img_projsKDS_jac_x
_StructureJacobianExpert_Cameras.__doc__="""
Function prototype for callable projection fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Does projection from P to X. 
"""

# Ideal cameras with no distortion
_libsbaprojs.img_projsKRTS_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                         ctypes.POINTER(Crsm),
                                         ctypes.POINTER(ctypes.c_int),
                                         ctypes.POINTER(ctypes.c_int),
                                         ctypes.POINTER(ctypes.c_double),
                                         ctypes.c_void_p]
_libsbaprojs.img_projsKRTS_x.restype = None
_libsbaprojs.img_projsKRTS_x.__str__ = "MotionStructureExpert_NoDistortion"
# Python handle
_MotionStructureExpert_NoDistortion = _libsbaprojs.img_projsKRTS_x
_MotionStructureExpert_NoDistortion.__doc__="""
Function prototype for callable projection func in expert mode sba:
void (*func) (double *p, struct sba_crsm *idxij, int *rcidxs, 
int *rcsubs, double *hx, void *adata)

Does projection from P to X. 
"""

_libsbaprojs.img_projsKRTS_jac_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                             ctypes.POINTER(Crsm),
                                             ctypes.POINTER(ctypes.c_int),
                                             ctypes.POINTER(ctypes.c_int),
                                             ctypes.POINTER(ctypes.c_double),
                                             ctypes.c_void_p]
_libsbaprojs.img_projsKRTS_jac_x.restype = None
_libsbaprojs.img_projsKRTS_jac_x.__str__ = "MotionStructureJacobianExpert_NoDistortion"
# Python handle
_MotionStructureJacobianExpert_NoDistortion = _libsbaprojs.img_projsKRTS_jac_x
_MotionStructureJacobianExpert_NoDistortion.__doc__="""
Function prototype for callable projection fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Does projection from P to X. 
"""

_libsbaprojs.img_projsKRT_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                        ctypes.POINTER(Crsm),
                                        ctypes.POINTER(ctypes.c_int),
                                        ctypes.POINTER(ctypes.c_int),
                                        ctypes.POINTER(ctypes.c_double),
                                        ctypes.c_void_p]
_libsbaprojs.img_projsKRT_x.restype = None
_libsbaprojs.img_projsKRT_x.__str__ = "MotionExpert_NoDistortion"
# Python handle
_MotionExpert_NoDistortion = _libsbaprojs.img_projsKRT_x
_MotionExpert_NoDistortion.__doc__="""
Function prototype for callable projection func in expert mode sba:
void (*func) (double *p, struct sba_crsm *idxij, int *rcidxs, 
int *rcsubs, double *hx, void *adata)

Does projection from P to X. 
"""

_libsbaprojs.img_projsKRT_jac_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                            ctypes.POINTER(Crsm),
                                            ctypes.POINTER(ctypes.c_int),
                                            ctypes.POINTER(ctypes.c_int),
                                            ctypes.POINTER(ctypes.c_double),
                                            ctypes.c_void_p]
_libsbaprojs.img_projsKRT_jac_x.restype = None
_libsbaprojs.img_projsKRT_jac_x.__str__ = "MotionJacobianExpert_NoDistortion"
# Python handle
_MotionJacobianExpert_NoDistortion = _libsbaprojs.img_projsKRT_jac_x
_MotionJacobianExpert_NoDistortion.__doc__="""
Function prototype for callable projection fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Does projection from P to X. 
"""

_libsbaprojs.img_projsKS_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                       ctypes.POINTER(Crsm),
                                       ctypes.POINTER(ctypes.c_int),
                                       ctypes.POINTER(ctypes.c_int),
                                       ctypes.POINTER(ctypes.c_double),
                                       ctypes.c_void_p]
_libsbaprojs.img_projsKS_x.restype = None
_libsbaprojs.img_projsKS_x.__str__ = "StructureExpert_NoDistortion"
# Python handle
_StructureExpert_NoDistortion = _libsbaprojs.img_projsKRT_x
_StructureExpert_NoDistortion.__doc__="""
Function prototype for callable projection func in expert mode sba:
void (*func) (double *p, struct sba_crsm *idxij, int *rcidxs, 
int *rcsubs, double *hx, void *adata)

Does projection from P to X. 
"""

_libsbaprojs.img_projsKS_jac_x.argtypes = [ctypes.POINTER(ctypes.c_double),
                                           ctypes.POINTER(Crsm),
                                           ctypes.POINTER(ctypes.c_int),
                                           ctypes.POINTER(ctypes.c_int),
                                           ctypes.POINTER(ctypes.c_double),
                                           ctypes.c_void_p]
_libsbaprojs.img_projsKS_jac_x.restype = None
_libsbaprojs.img_projsKS_jac_x.__str__ = "StructureJacobianExpert_NoDistortion"
# Python handle
_StructureJacobianExpert_NoDistortion = _libsbaprojs.img_projsKS_jac_x
_StructureJacobianExpert_NoDistortion.__doc__="""
Function prototype for callable projection fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Does projection from P to X. 
"""







# Simple mode
_libsbaprojs.img_projRTS.argtypes = [ctypes.c_int, # j
                                     ctypes.c_int, # i
                                     ctypes.POINTER(ctypes.c_double), # *bi
                                     ctypes.POINTER(ctypes.c_double), # *xij
                                     ctypes.c_void_p]
_libsbaprojs.img_projRTS.restype = None
_libsbaprojs.img_projRTS.__str__ = "MotionsStructureSimple_NoCameras"
# Python handle
_MotionStructureSimple_NoCameras = _libsbaprojs.img_projRTS
_MotionStructureSimple_NoCameras.__doc__="""
Function prototype for callable projection func in simple mode sba:
void (*proj) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""
_libsbaprojs.img_projRT.argtypes = [ctypes.c_int, # j
                                    ctypes.c_int, # i
                                    ctypes.POINTER(ctypes.c_double), # *bi
                                    ctypes.POINTER(ctypes.c_double), # *xij
                                    ctypes.c_void_p]
_libsbaprojs.img_projRT.restype = None
_libsbaprojs.img_projRT.__str__ = "MotionSimple_NoCameras"
# Python handle
_MotionSimple_NoCameras = _libsbaprojs.img_projRT
_MotionSimple_NoCameras.__doc__="""
Function prototype for callable projection func in simple mode sba:
void (*proj) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""
_libsbaprojs.img_projS.argtypes = [ctypes.c_int, # j
                                   ctypes.c_int, # i
                                   ctypes.POINTER(ctypes.c_double), # *bi
                                   ctypes.POINTER(ctypes.c_double), # *xij
                                   ctypes.c_void_p]
_libsbaprojs.img_projS.restype = None
_libsbaprojs.img_projS.__str__ = "StructureSimple_NoCameras"
# Python handle
_StructureSimple_NoCameras = _libsbaprojs.img_projS
_StructureSimple_NoCameras.__doc__="""
Function prototype for callable projection func in simple mode sba:
void (*proj) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""

_libsbaprojs.img_projKDRTS.argtypes = [ctypes.c_int, # j
                                       ctypes.c_int, # i
                                       ctypes.POINTER(ctypes.c_double), # *bi
                                       ctypes.POINTER(ctypes.c_double), # *xij
                                       ctypes.c_void_p]
_libsbaprojs.img_projKDRTS.restype = None
_libsbaprojs.img_projKDRTS.__str__ = "MotionsStructureSimple_Cameras"
# Python handle
_MotionStructureSimple_Cameras = _libsbaprojs.img_projKDRTS
_MotionStructureSimple_Cameras.__doc__="""
Function prototype for callable projection func in simple mode sba:
void (*proj) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""
_libsbaprojs.img_projKDRT.argtypes = [ctypes.c_int, # j
                                      ctypes.c_int, # i
                                      ctypes.POINTER(ctypes.c_double), # *bi
                                      ctypes.POINTER(ctypes.c_double), # *xij
                                      ctypes.c_void_p]
_libsbaprojs.img_projKDRT.restype = None
_libsbaprojs.img_projKDRT.__str__ = "MotionSimple_Cameras"
# Python handle
_MotionSimple_Cameras = _libsbaprojs.img_projKDRT
_MotionSimple_Cameras.__doc__="""
Function prototype for callable projection func in simple mode sba:
void (*proj) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""
_libsbaprojs.img_projKDS.argtypes = [ctypes.c_int, # j
                                     ctypes.c_int, # i
                                     ctypes.POINTER(ctypes.c_double), # *bi
                                     ctypes.POINTER(ctypes.c_double), # *xij
                                     ctypes.c_void_p]
_libsbaprojs.img_projKDS.restype = None
_libsbaprojs.img_projKDS.__str__ = "StructureSimple_Cameras"
# Python handle
_StructureSimple_Cameras = _libsbaprojs.img_projKDS
_StructureSimple_Cameras.__doc__="""
Function prototype for callable projection func in simple mode sba:
void (*proj) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""

_libsbaprojs.img_projKRTS.argtypes = [ctypes.c_int, # j
                                      ctypes.c_int, # i
                                      ctypes.POINTER(ctypes.c_double), # *bi
                                      ctypes.POINTER(ctypes.c_double), # *xij
                                      ctypes.c_void_p]
_libsbaprojs.img_projKRTS.restype = None
_libsbaprojs.img_projKRTS.__str__ = "MotionsStructureSimple_NoDistortion"
# Python handle
_MotionStructureSimple_NoDistortion = _libsbaprojs.img_projKRTS
_MotionStructureSimple_NoDistortion.__doc__="""
Function prototype for callable projection func in simple mode sba:
void (*proj) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""
_libsbaprojs.img_projKRT.argtypes = [ctypes.c_int, # j
                                     ctypes.c_int, # i
                                     ctypes.POINTER(ctypes.c_double), # *bi
                                     ctypes.POINTER(ctypes.c_double), # *xij
                                     ctypes.c_void_p]
_libsbaprojs.img_projKRT.restype = None
_libsbaprojs.img_projKRT.__str__ = "MotionSimple_NoDistortion"
# Python handle
_MotionSimple_NoDistortion = _libsbaprojs.img_projKRT
_MotionSimple_NoDistortion.__doc__="""
Function prototype for callable projection func in simple mode sba:
void (*proj) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""
_libsbaprojs.img_projKS.argtypes = [ctypes.c_int, # j
                                    ctypes.c_int, # i
                                    ctypes.POINTER(ctypes.c_double), # *bi
                                    ctypes.POINTER(ctypes.c_double), # *xij
                                    ctypes.c_void_p]
_libsbaprojs.img_projKS.restype = None
_libsbaprojs.img_projKS.__str__ = "StructureSimple_NoDistortion"
# Python handle
_StructureSimple_NoDistortion = _libsbaprojs.img_projKS
_StructureSimple_NoDistortion.__doc__="""
Function prototype for callable projection func in simple mode sba:
void (*proj) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""

_libsbaprojs.img_projRTS.argtypes = [ctypes.c_int, # j
                                     ctypes.c_int, # i
                                     ctypes.POINTER(ctypes.c_double), # *bi
                                     ctypes.POINTER(ctypes.c_double), # *xij
                                     ctypes.c_void_p]
_libsbaprojs.img_projRTS.restype = None
_libsbaprojs.img_projRTS.__str__ = "MotionsStructureSimple_NoCameras"
# Python handle
_MotionStructureSimple_NoCameras = _libsbaprojs.img_projRTS
_MotionStructureSimple_NoCameras.__doc__="""
Function prototype for callable projection func in simple mode sba:
void (*proj) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""
_libsbaprojs.img_projRT.argtypes = [ctypes.c_int, # j
                                    ctypes.c_int, # i
                                    ctypes.POINTER(ctypes.c_double), # *bi
                                    ctypes.POINTER(ctypes.c_double), # *xij
                                    ctypes.c_void_p]
_libsbaprojs.img_projRT.restype = None
_libsbaprojs.img_projRT.__str__ = "MotionSimple_NoCameras"
# Python handle
_MotionSimple_NoCameras = _libsbaprojs.img_projRT
_MotionSimple_NoCameras.__doc__="""
Function prototype for callable projection func in simple mode sba:
void (*proj) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""
_libsbaprojs.img_projS.argtypes = [ctypes.c_int, # j
                                   ctypes.c_int, # i
                                   ctypes.POINTER(ctypes.c_double), # *bi
                                   ctypes.POINTER(ctypes.c_double), # *xij
                                   ctypes.c_void_p]
_libsbaprojs.img_projS.restype = None
_libsbaprojs.img_projS.__str__ = "StructureSimple_NoCameras"
# Python handle
_StructureSimple_NoCameras = _libsbaprojs.img_projS
_StructureSimple_NoCameras.__doc__="""
Function prototype for callable projection func in simple mode sba:
void (*proj) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""

_libsbaprojs.img_projKDRTS_jac.argtypes = [ctypes.c_int, # j
                                           ctypes.c_int, # i
                                           ctypes.POINTER(ctypes.c_double), # *bi
                                           ctypes.POINTER(ctypes.c_double), # *xij
                                           ctypes.c_void_p]
_libsbaprojs.img_projKDRTS_jac.restype = None
_libsbaprojs.img_projKDRTS_jac.__str__ = "MotionsStructureJacobianSimple_Cameras"
# Python handle
_MotionStructureJacobianSimple_Cameras = _libsbaprojs.img_projKDRTS_jac
_MotionStructureJacobianSimple_Cameras.__doc__="""
Function prototype for callable projection fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a Jacobian from 
Python into C.
"""
_libsbaprojs.img_projKDRT_jac.argtypes = [ctypes.c_int, # j
                                          ctypes.c_int, # i
                                          ctypes.POINTER(ctypes.c_double), # *bi
                                          ctypes.POINTER(ctypes.c_double), # *xij
                                          ctypes.c_void_p]
_libsbaprojs.img_projKDRT_jac.restype = None
_libsbaprojs.img_projKDRT_jac.__str__ = "MotionJacobianSimple_Cameras"
# Python handle
_MotionJacobianSimple_Cameras = _libsbaprojs.img_projKDRT_jac
_MotionJacobianSimple_Cameras.__doc__="""
Function prototype for callable projection fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""
_libsbaprojs.img_projKDS_jac.argtypes = [ctypes.c_int, # j
                                         ctypes.c_int, # i
                                         ctypes.POINTER(ctypes.c_double), # *bi
                                         ctypes.POINTER(ctypes.c_double), # *xij
                                         ctypes.c_void_p]
_libsbaprojs.img_projKDS_jac.restype = None
_libsbaprojs.img_projKDS_jac.__str__ = "StructureJacobianSimple_Cameras"
# Python handle
_StructureJacobianSimple_Cameras = _libsbaprojs.img_projKDS_jac
_StructureJacobianSimple_Cameras.__doc__="""
Function prototype for callable projection fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""

_libsbaprojs.img_projKRTS_jac.argtypes = [ctypes.c_int, # j
                                          ctypes.c_int, # i
                                          ctypes.POINTER(ctypes.c_double), # *bi
                                          ctypes.POINTER(ctypes.c_double), # *xij
                                          ctypes.c_void_p]
_libsbaprojs.img_projKRTS_jac.restype = None
_libsbaprojs.img_projKRTS_jac.__str__ = "MotionsStructureJacobianSimple_NoDistortion"
# Python handle
_MotionStructureJacobianSimple_NoDistortion = _libsbaprojs.img_projKRTS_jac
_MotionStructureJacobianSimple_NoDistortion.__doc__="""
Function prototype for callable projection fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""
_libsbaprojs.img_projKRT_jac.argtypes = [ctypes.c_int, # j
                                         ctypes.c_int, # i
                                         ctypes.POINTER(ctypes.c_double), # *bi
                                         ctypes.POINTER(ctypes.c_double), # *xij
                                         ctypes.c_void_p]
_libsbaprojs.img_projKRT_jac.restype = None
_libsbaprojs.img_projKRT_jac.__str__ = "MotionJacobianSimple_NoDistortion"
# Python handle
_MotionJacobianSimple_NoDistortion = _libsbaprojs.img_projKRT_jac
_MotionJacobianSimple_NoDistortion.__doc__="""
Function prototype for callable projection fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""
_libsbaprojs.img_projKS_jac.argtypes = [ctypes.c_int, # j
                                        ctypes.c_int, # i
                                        ctypes.POINTER(ctypes.c_double), # *bi
                                        ctypes.POINTER(ctypes.c_double), # *xij
                                        ctypes.c_void_p]
_libsbaprojs.img_projKS_jac.restype = None
_libsbaprojs.img_projKS_jac.__str__ = "StructureJacobianSimple_NoDistortion"
# Python handle
_StructureJacobianSimple_NoDistortion = _libsbaprojs.img_projKS_jac
_StructureJacobianSimple_NoDistortion.__doc__="""
Function prototype for callable projection fjac in simple mode sba:
void (*fjac) (int j, int i, double *bi, double *xij, void *adata)

Use to wrap a Python function when sending a projection function from 
Python into C.
"""






def whichProjection(options):
    """
    Dispatcher for providing the correct projection function from C

    input:
    options - Options object specifying sba options

    returns:
    a Python pointer to a C function (ctypes.funcptr) for sba projection.
    This can be passed to an sba driver, eg sba_levmar_motstr_x

    Raises SbaError if options are not recognized
    """
    #logging.debug("projections.whichProjection() called with {0}".format(options))
    if options.expert:
        if options.motstruct==sba.options.OPTS_MOTSTRUCT:
            if options.camera==sba.options.OPTS_CAMS:
                #return _libsbaprojs.img_projsKDRTS_x
                return _MotionStructureExpert_Cameras
            elif options.camera==sba.options.OPTS_CAMS_NODIST:
                #return _libsbaprojs.img_projsKRTS_x
                return _MotionStructureExpert_NoDistortion
            elif options.camera==sba.options.OPTS_NO_CAMS:
                return _MotionStructureExpert_NoCameras
            else:
                raise SbaError("Unrecognized projection option")
        elif options.motstruct==sba.options.OPTS_MOT:
            if options.camera==sba.options.OPTS_CAMS:
                return _MotionExpert_Cameras
            elif options.camera==sba.options.OPTS_CAMS_NODIST:
                return _MotionExpert_NoDistortion
            elif options.camera==sba.options.OPTS_NO_CAMS:
                return _MotionExpert_NoCameras
            else:
                raise SbaError("Unrecognized projection option")
        elif options.motstruct==sba.options.OPTS_STRUCT:
            if options.camera==sba.options.OPTS_CAMS:
                return _StructureExpert_Cameras
            elif options.camera==sba.options.OPTS_CAMS_NODIST:
                return _StructureExpert_NoDistortion
            elif options.camera==sba.options.OPTS_NO_CAMS:
                return _StructureExpert_NoCameras
            else:
                raise SbaError("Unrecognized projection option")
    else: # simple mode
        if options.motstruct==sba.options.OPTS_MOTSTRUCT:
            if options.camera==sba.options.OPTS_CAMS:
                return _MotionStructureSimple_Cameras
            elif options.camera==sba.options.OPTS_CAMS_NODIST:
                return _MotionStructureSimple_NoDistortion
            elif options.camera==sba.options.OPTS_NO_CAMS:
                return _MotionStructureSimple_NoCameras
            else:
                raise SbaError("Unrecognized projection option")
        elif options.motstruct==sba.options.OPTS_MOT:
            if options.camera==sba.options.OPTS_CAMS:
                return _MotionSimple_Cameras
            elif options.camera==sba.options.OPTS_CAMS_NODIST:
                return _MotionSimple_NoDistortion
            elif options.camera==sba.options.OPTS_NO_CAMS:
                return _MotionSimple_NoCameras
            else:
                raise SbaError("Unrecognized projection option")
        elif options.motstruct==sba.options.OPTS_STRUCT:
            if options.camera==sba.options.OPTS_CAMS:
                return _StructureSimple_Cameras
            elif options.camera==sba.options.OPTS_CAMS_NODIST:
                return _StructureSimple_NoDistortion
            elif options.camera==sba.options.OPTS_NO_CAMS:
                return _StructureSimple_NoCameras
            else:
                raise SbaError("Unrecognized projection option")

def whichJacobian(options):
    """
    Dispatcher for providing the correct Jacobian function

    input:
    options - Options object specifying sba options

    returns:
    a Python pointer to a C function (ctypes.funcptr) for sba Jacobian 
    This can be passed to an sba driver, eg sba_levmar_motstr_x

    Raises SbaError if options are not recognized
    """
    #logging.debug("projections.whichJacobian() called with {0}".format(options))
    if options.expert:
        if options.analjac:
            if options.motstruct==sba.options.OPTS_MOTSTRUCT:
                if options.camera==sba.options.OPTS_CAMS:
                    return _MotionStructureJacobianExpert_Cameras
                elif options.camera==sba.options.OPTS_CAMS_NODIST:
                    return _MotionStructureJacobianExpert_NoDistortion
                elif options.camera==sba.options.OPTS_NO_CAMS:
                    return _MotionStructureJacobianExpert_NoCameras
                else:
                    raise SbaError("Unrecognized Jacobian option")
            elif options.motstruct==sba.options.OPTS_MOT:
                if options.camera==sba.options.OPTS_CAMS:
                    return _MotionJacobianExpert_Cameras
                elif options.camera==sba.options.OPTS_CAMS_NODIST:
                    return _MotionJacobianExpert_NoDistortion
                elif options.camera==sba.options.OPTS_NO_CAMS:
                    return _MotionJacobianExpert_NoCameras
                else:
                    raise SbaError("Unrecognized Jacobian option")
            elif options.motstruct==sba.options.OPTS_STRUCT:
                if options.camera==sba.options.OPTS_CAMS:
                    return _StructureJacobianExpert_Cameras
                elif options.camera==sba.options.OPTS_CAMS_NODIST:
                    return _StructureJacobianExpert_NoDistortion
                elif options.camera==sba.options.OPTS_NO_CAMS:
                    return _StructureJacobianExpert_NoCameras
                else:
                    raise SbaError("Unrecognized Jacobian option")
        else:
            return None
    else: # simple mode
        if options.analjac:
            if options.motstruct==sba.options.OPTS_MOTSTRUCT:
                if options.camera==sba.options.OPTS_CAMS:
                    return _MotionStructureJacobianSimple_Cameras
                elif options.camera==sba.options.OPTS_CAMS_NODIST:
                    return _MotionStructureJacobianSimple_NoDistortion
                elif options.camera==sba.options.OPTS_NO_CAMS:
                    return _MotionStructureJacobianSimple_NoCameras
                else:
                    raise SbaError("Unrecognized Jacobian option")
            elif options.motstruct==sba.options.OPTS_MOT:
                if options.camera==sba.options.OPTS_CAMS:
                    return _MotionJacobianSimple_Cameras
                elif options.camera==sba.options.OPTS_CAMS_NODIST:
                    return _MotionJacobianSimple_NoDistortion
                elif options.camera==sba.options.OPTS_NO_CAMS:
                    return _MotionJacobianSimple_NoCameras
                else:
                    raise SbaError("Unrecognized Jacobian option")
            elif options.motstruct==sba.options.OPTS_STRUCT:
                if options.camera==sba.options.OPTS_CAMS:
                    return _StructureJacobianSimple_Cameras
                elif options.camera==sba.options.OPTS_CAMS_NODIST:
                    return _StructureJacobianSimple_NoDistortion
                elif options.camera==sba.options.OPTS_NO_CAMS:
                    return _StructureJacobianSimple_NoCameras
                else:
                    raise SbaError("Unrecognized Jacobian option")
            else:
                return None







#!/usr/bin/env python
"""
info.py
Dennis Evangelista, 2013
Python implementation of Lourakis' sba code and parts of demo
This is for handling output info 
"""

from __future__ import absolute_import
from __future__ import print_function
import ctypes
import numpy as np
import sba.options

# Provides readable reason for algorithm termination
INFO_REASONS = {1:"stopped by small gradient J^T e", 
                2:"stopped by small dp",
                3:"stopped by itmax",
                4:"stopped by small relative reduction in e_2",
                5:"too many attempts to increase damping, restart with increased mu",
                6:"stopped by small e_2",
                7:"error, stopped by invalid NaN or Inf func values",
                0:"no reason, sometimes it just happens"}

INFO_SIZE = 10 # Size of info array double [INFO SIZE]

CInfoType = ctypes.c_double*INFO_SIZE

class Information(object):
    """
    Python friendly class for representing information output by sba

    attributes:
    eps0 - LM parameter, initial value
    epsF - LM parameter, final value
    JTeps - J^Teps_inf LM parameter
    DP - LM parameter
    mu_max - LM parameter
    iterations - number of iterations
    reason - for termination. see why() method 
    fevals - number of projection function evaluations
    jevals - number of Jacobian evaluations
    nsystems - number of linear systems solved
    
    infoarray - access to 10 element numpy array 

    additional attributes (copied in during SparseBundleAdjust)
    nprojs - number of projections (3D point mapped to a 2D point in 1 image)
    nvars - number of free variables adjusted
    method - method used
    analjac - analytic (True) or approximate (False)
    havecovx - with (True) or without (False)
    havedistortion, ncdist
    fixedcal, nccalib
    t0 - start time of last sparse bundle adjustment
    tf - end time of last sparse bundle adjustment
    """
    def __init__(self,infoarray=None):
        if np.all(infoarray): # constructor called with input
            self.__eps0 = infoarray[0]
            self.__epsF = infoarray[1]
            self.__JTeps = infoarray[2]
            self.__DP = infoarray[3]
            self.__mu_max = infoarray[4]
            self.__iterations = infoarray[5]
            self.__reason = infoarray[6]
            self.__fevals = infoarray[7]
            self.__jevals = infoarray[8]
            self.__nsystems = infoarray[9]
            self.__infoarray = np.array(infoarray,dtype=np.double)
        else: # constructor called with no input
            self.__eps0 = 0.
            self.__epsF = 0.
            self.__JTeps = 0.
            self.__DP = 0.
            self.__mu_max = 0.
            self.__iterations = 0
            self.__reason = 0
            self.__fevals = 0
            self.__jevals = 0
            self.__nsystems = 0
            self.__infoarray = np.array([self.__eps0,self.__epsF,self.__JTeps,
                                         self.__DP,self.__mu_max,
                                         self.__iterations,
                                         self.__reason,self.__fevals,
                                         self.__jevals,
                                         self.__nsystems],
                                        dtype=np.double)

            # additional attributes
            self.__n=0
            self.__ncameras = 0
            self.__nprojs = 0 # number of projections of 3D->2D in an image
            self.__nvars = 0 # number of free variables adjusted
            self.__method = "not defined" # e.g. MotionStructureExpert_Cameras
            self.__expert = False 
            self.__analjac = False # have analytic Jacobian
            self.__havecovx = False # have input covx for image poitns
            self.__havedist = False # using distortion?
            self.__ncdist = 0 # if so, how many distortion coeffs fixed? 
            self.__fixedcal = False # using fixed cal for all cameras? 
            self.__nccalib = 0 # if not, how many intrinsics are fixed?  
            self.__t0 = 0. # start time of last sparse bundle adjustment
            self.__tf = 0. # end time of last sparse bundle adjustment
            
            
    def why(self):
        """What is the sound of one hand clapping?"""
        return INFO_REASONS[int(self.__reason)]

    def toC(self):
        """Provides C format info array as double *"""
        return CInfoType(self.__infoarray[0],self.__infoarray[1],
                         self.__infoarray[2],self.__infoarray[3],
                         self.__infoarray[4],self.__infoarray[5],
                         self.__infoarray[6],self.__infoarray[7],
                         self.__infoarray[8],self.__infoarray[9])

    def fromC(self,cinfo):
        temp = np.array([cinfo[0],cinfo[1],cinfo[2],cinfo[3],cinfo[4],
                         cinfo[5],cinfo[6],cinfo[7],cinfo[8],cinfo[9]],
                        dtype=np.double)
        self.infoarray=temp

    def _geteps0(self):
        return self.__eps0
    def _seteps0(self,value):
        self.__eps0 = value
        self.__infoarray[0] = self.__eps0
    eps0 = property(_geteps0,_seteps0,doc="eps0 initial error")

    def _getepsF(self):
        return self.__epsF
    def _setepsF(self,value):
        self.__epsF = value
        self.__infoarray[1] = self.__epsF
    epsF = property(_getepsF,_setepsF,doc="epsF final error")

    def _getJTeps(self):
        return self.__JTeps
    def _setJTeps(self,value):
        self.__JTeps = value
        self.__infoarray[2] = self.__JTeps
    JTeps = property(_getJTeps,_setJTeps,doc="J^Teps_inf LM parameter")

    def _getDP(self):
        return self.__DP
    def _setDP(self,value):
        self.__DP = value
        self.__infoarray[3] = self.__DP
    DP = property(_getDP,_setDP,doc="DP LM parameter")

    def _getmu_max(self):
        return self.__mu_max
    def _setmu_max(self,value):
        self.__mu_max = value
        self.__infoarray[4] = self.__mu_max
    mu_max = property(_getmu_max,_setmu_max,doc="mu_max LM parameter")

    def _getiterations(self):
        return int(self.__iterations)
    def _setiterations(self,value):
        self.__iterations = value
        self.__infoarray[5] = self.__iterations
    iterations = property(_getiterations,_setiterations,doc="number of iterations")

    def _getreason(self):
        return int(self.__reason)
    def _setreason(self,value):
        self.__reason = value
        self.__infoarray[6] = self.__reason
    reason = property(_getreason,_setreason,doc="reason for stopping, see also INFO_REASONS")

    def _getfevals(self):
        return int(self.__fevals)
    def _setfevals(self,value):
        self.__fevals = value
        self.__infoarray[7] = self.__fevals
    fevals = property(_getfevals,_setfevals,doc="number of projection function evals")

    def _getjevals(self):
        return int(self.__jevals)
    def _setjevals(self,value):
        self.__jevals = value
        self.__infoarray[8] = self.__jevals
    jevals = property(_getjevals,_setjevals,doc="number of Jacobian function evals")

    def _getnsystems(self):
        return self.__nsystems
    def _setnsystems(self,value):
        self.__nsystems = value
        self.__infoarray[9] = self.__nsystems
    nsystems = property(_getnsystems,_setnsystems,doc="number of linear systems solved")

    def _getinfoarray(self):
        return self.__infoarray
    def _setinfoarray(self,newinfoarray):
        self.__infoarray = np.array(newinfoarray,dtype=np.double)
        self.__eps0 = self.__infoarray[0]
        self.__epsF = self.__infoarray[1]
        self.__JTeps = self.__infoarray[2]
        self.__DP = self.__infoarray[3]
        self.__mu_max = self.__infoarray[4]
        self.__iterations = self.__infoarray[5]
        self.__reason = self.__infoarray[6]
        self.__fevals = self.__infoarray[7]
        self.__jevals = self.__infoarray[8]
        self.__nsystems = self.__infoarray[9]
    infoarray = property(_getinfoarray,_setinfoarray,doc="10 element info array")

    def __str__(self):
        """Human readable representation of sba.Information()"""
        return "<sba.Information() object, last stopped for {0}>".format(self.why()) 

    def _getn(self):
        return self.__n
    def _setn(self,value):
        self.__n = value
    n = property(_getn,_setn,doc="Number of 3D points")

    def _getncameras(self):
        return self.__ncameras
    def _setncameras(self,value):
        self.__ncameras = value
    ncameras = property(_getncameras,_setncameras,doc="Number of cameras")

    def _getnprojs(self):
        return self.__nprojs
    def _setnprojs(self,value):
        self.__nprojs = value
    nprojs = property(_getnprojs,_setnprojs,
                      doc="""
    number of projections of 3D-2D, over all cams. In SparseBundleAdjust, this is computed
    as the sum of points.vmask""")

    def _getnvars(self):
        return self.__nvars
    def _setnvars(self,value):
        self.__nvars = value
    nvars = property(_getnvars,_setnvars,
                     doc="""
    number of free variables adjusted.  In SparseBundleAdjust, this is computed as:
    ncameras*cnp+npts3d*pnp is using motion and structure
    ncameras*cnp if using motion only
    npts3d*pnp if using structure only""")

    def _getmethod(self):
        return self.__method
    def _setmethod(self,value):
        self.__method = value
    method = property(_getmethod,_setmethod,
                      doc="""e.g. MotionStructureExpert_Cameras.  In SparseBundleAdjust, 
    this is found using the dispatcher's projection.__str__ method.""")

    def _getexpert(self):
        return self.__expert
    def _setexpert(self,value):
        self.__expert = value
    expert = property(_getexpert,_setexpert,
                      doc="used expert mode (True) or simple mode (False)?")

    def _getanaljac(self):
        return self.__analjac
    def _setanaljac(self,value):
        self.__analjac = value
    analjac = property(_getanaljac,_setanaljac,
                       doc="used analytic Jacobian or approximate? from options")

    def _gethascovx(self):
        return self.__havecovx
    def _sethascovx(self,value):
        self.__havecovx = value
    hascovx = property(_gethascovx,_sethascovx,
                        doc="has input image point covariances? from points")

    def _gethavedist(self):
        return self.__havedist
    def _sethavedist(self,value):
        self.__havedist = value
    hasDistortion = property(_gethavedist,_sethavedist,
                             doc="had distortion in camera model? from camera")
    
    def _getncdist(self):
        return self.__ncdist
    def _setncdist(self,value):
        self.__ncdist = value
    ncdist = property(_getncdist,_setncdist,
                      doc="if distortion was used, how many dist coeffs were fixed? from options")

    def _getfixedcal(self):
        return self.__fixedcal
    def _setfixedcal(self,value):
        self.__fixedcal = value
    hasFixedCal = property(_getfixedcal, _setfixedcal, 
                           doc="using fixed cal for all cameras? from options")

    def _getnccalib(self):
        return self.__nccalib
    def _setnccalib(self,value):
        self.__nccalib = value
    nccalib = property(_getnccalib, _setnccalib,
                       doc="if camera calibrations varied, how many intrinsics were fixed? from options")
    
    def _gett0(self):
        return self.__t0
    def _sett0(self,value):
        self.__t0 = value
    t0 = property(_gett0,_sett0,
                  doc="""start time of last sparse bundle adjustment.
    from time.clock() at start of SparseBundleAdjust""")

    def _gettf(self):
        return self.__tf
    def _settf(self,value):
        self.__tf = value
    tf = property(_gettf,_settf,
                  doc="""end time of last sparse bundle adjustment. 
    from time.clock() at end of SparseBundleAdjust""")

    def _fromInput(cls,cameras,points,options):
        """
        Alternate constructor (a class method) to create default
        Information object with values drawn from input objects
        cameras, points, and options.
        """
        newinfo = cls()

        newinfo.n = int(points.n)
        newinfo.ncameras = int(cameras.ncameras)
        newinfo.nprojs = int(np.sum(np.sum(points.vmask)))

        if options.motstruct == sba.options.OPTS_MOTSTRUCT:
            newinfo.nvars = int(cameras.ncameras*options.cnp+points.n*options.pnp)
        elif options.motstruct == sba.options.OPTS_MOT:
            newinfo.nvars = int(cameras.ncameras*options.cnp)
        elif options.motstruct == sba.options.OPTS_STRUCT:
            newinfo.nvars = int(points.n*options.pnp)

        newinfo.method = sba.routines.whichRoutine(options).__str__
        # do this during SparseBundleAdjust?

        newinfo.expert = options.expert
        newinfo.analjac = options.analjac
        newinfo.hascovx = points.hascovx
        newinfo.hasDistortion = options.hasDistortion
        newinfo.ncdist = options.ncdist
        newinfo.hasFixedCal = options.hasFixedCal
        newinfo.nccalib = options.nccalib
        return newinfo
    fromInput = classmethod(_fromInput)


    # pretty print methods
    def printResults(self):
        """Print results to stdout"""
        print("")
        print("SBA using {0} 3D pts, {1} frames and {2} image projections, {3} variables".format(self.n, self.ncameras, self.nprojs, self.nvars))
        print("")
        print("Method {0}, {expert} driver, {analjac} Jacobian, {hascovx} covariances, {hasDistortion} distortion{ncdist}, {fixedcal} intrinsics{nccalib}".format(self.method,expert="expert" if self.expert else "simple",analjac="analytic" if self.analjac else "approximate",hascovx="with" if self.hascovx else "without",hasDistortion="variable" if self.hasDistortion else "without",ncdist=" ({0} fixed)".format(self.ncdist) if self.hasDistortion else "",fixedcal="fixed" if self.hasFixedCal else "variable",nccalib=" ({0} fixed)".format(self.nccalib) if not self.hasFixedCal else ""))
        print("")
        print("SBA returned {0} in {1} iter, reason {2}, error {3:f} [initial {4:f}, {5}/{6} func/fjac evals, {7} lin. systems".format(self.n,int(self.iterations),int(self.reason),self.epsF/float(self.nprojs),self.eps0/float(self.nprojs),int(self.fevals),int(self.jevals),int(self.nsystems)))
        print("(Reason {0} is {1})".format(int(self.reason),self.why()))
        print("Elapsed time: {0} seconds, {1} msecs".format(self.tf-self.t0,(self.tf-self.t0)*1000.))


        



from setuptools import setup
#from distutils.core import setup # no longer recommended

import os

setup(
    name='sba',
    version='1.6.8.1',
    packages=['sba'],
    scripts=['bin/eucsbademo.py'],

    # dependencies
    #include_package_data=True,
    #install_requires=[
        #"quaternions (>=0.0)" no longer required
        #"numpy>=1.12",
    #],
    dependency_links=[
        'http://www.ics.forth.gr/~lourakis/sba', # lourakis C library
        'https://bitbucket.org/devangel77b/libsbaprojs' # projections
    ],
    #zip_safe=False,

    # metadata for upload to PyPI
    author='Dennis Evangelista',
    author_email='devangel77b@gmail.com',
    description="wrapper for Lourakis' sparse bundle adjustment C library",
    #license='GNU GPLv3',
    #keywords='SBA, sparse bundle adjustment, calibration, camera, camera calibration, photogrammetry',
    #url='hg+https://bitbucket.org/devangel77b/python-sba',
    #url='https://bitbucket.org/devangel77b/python-sba',

    long_description=open(os.path.join(os.path.dirname(__file__),'README.md'),'r').read(),
    #classifiers=[
    #    'Development Status :: 3 - Alpha',
    #    'Programming Language :: Python :: 2.7',
    #    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    #    'Topic :: Scientific/Engineering',
    #    'Topic :: Multimedia :: Video',
    #    'Topic :: Multimedia :: Graphics',
    #    'Topic :: Multimedia :: Graphics :: 3D Modeling',
    #    'Topic :: Multimedia :: Graphics :: Capture :: Digital Camera',
    #    'Operating System :: POSIX :: Linux',
    #    'Operating System :: OS Independent',
    #    'Intended Audience :: Science/Research'
    #    ],
)



#!/usr/bin/env python
"""
This is a Python clone of the Lourakis eucsbademo program used 
to test the sba installation is working correctly. 

D Evangelista 2015ish
"""


import argparse
import logging
import numpy as np
import sba

__version__ = sba.__version__



def sbaDriver(camname,ptsname,intrname=None,camoname=None,ptsoname=None):
    """
    Python port of eucsba demo driver portion of Lourakis' code
    """
    logging.debug("sbaDriver() {0} {1}".format(camname,ptsname))

    if intrname is None:
        cameras = sba.Cameras.fromTxt(camname) # Load cameras from file
    else:
        # Intrinsic file for all identical cameras not yet implemented
        logging.warning("Fixed intrinsic parameters not yet implemented")
        cameras = sba.Cameras.fromTxtWithIntr(camname,intrname)
    points = sba.Points.fromTxt(ptsname,cameras.ncameras) # Load points

    options = sba.Options.fromInput(cameras,points)
    #options.camera = sba.OPTS_CAMS_NODIST # only use this if passing in 5+3+3
    options.nccalib=sba.OPTS_FIX2_INTR # fix all intrinsics
    # If you wish to fix the intrinsics do so here by setting options
    options.ncdist = sba.OPTS_FIX5_DIST # fix all distortion coeffs

    newcameras,newpoints,info = sba.SparseBundleAdjust(cameras,points,options)
    info.printResults()

    if camoname:
        newcameras.toTxt(camoname)
    if ptsoname:
        newpoints.toTxt(ptsoname)

    # other ways to save output:
    #  - all can be pickled 
    #  - save options to yaml (and load from)?






if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Sparse bundle adjustment using Lourakis' sba library")

    parser.add_argument("camname",
                        help="fx px py AR s r2 r4 t1 t2 r6 q0 qi qj qk tx ty tz")
    parser.add_argument("ptsname",
                        help="X Y Z Nvisible Cam0 x0 y0 Cam1 x1 y1")

    parser.add_argument("camo",nargs="?",default=None,help="Output filename for cameras txt")
    parser.add_argument("ptso",nargs="?",default=None,help="Output filename for 3D points txt")

    parser.add_argument("--intrname",help="Input intrinsics, not implemented")
    parser.add_argument("-v","--verbose",action="store_true",
                        help="Toggle verbosity")
    parser.add_argument("--version",action="version",
                        version = "%(prog)s {0}".format(__version__))
    args = parser.parse_args()

    if args.verbose:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    sbaDriver(args.camname,args.ptsname,camoname=args.camo,ptsoname=args.ptso)



    



#!/usr/bin/env python
"""
points.py
Dennis Evangelista, 2013
"""

from __future__ import absolute_import
import logging
import numpy as np
import ctypes
from six.moves import range

class Points(object):
    def __init__(self,B,X,vmask):
        self.__B=np.array(B,dtype=np.double)
        self.__n = self.__B.shape[0]
        self.__pnp = self.__B.shape[1]
        assert(self.__pnp==3)
        
        self.__X=np.array(X,dtype=np.double)
        self.__ncameras = self.__X.shape[1]
        self.__mnp = self.__X.shape[2]
        assert(self.__mnp==2)
        assert(self.__X.shape[0]==self.__n)

        self.__vmask=np.array(vmask,dtype=np.byte)
        assert(self.__vmask.shape==(self.__n,self.__ncameras))

        self.__covx = None

    def _getB(self):
        return self.__B
    def _setB(self,value):
        self.__B=np.array(value,dtype=np.double)
        self.__n = self.__B.shape[0]
        self.__pnp = self.__B.shape[1]
        assert(self.__pnp==3)
    B = property(_getB,_setB,doc="B is the 3D worldPoints array")

    def _getX(self):
        return self.__X
    def _setX(self,value):
        self.__X=np.array(value,dtype=np.double)
        self.__ncameras = self.__X.shape[1]
        self.__mnp = self.__X.shape[2]
        assert(self.__mnp==2)
    X = property(_getX,_setX,doc="X is the 2D imagePoints nonmasked array")
    
    def _getcovx(self):
        return self.__covx
    def _setcovx(self,value):
        self.__covx = value
    covx = property(_getcovx,_setcovx,doc="covx is not used and should be None")
    def covxToC(self):
        if self.__covx == None:
            return ctypes.POINTER(ctypes.c_double)()
        else:
            return self.__covx.ctypes.data_as(ctypes.POINTER(ctypes.c_double))
        
    def _getvmask(self):
        return self.__vmask
    def _setvmask(self,value):
        self.__vmask=np.array(value,dtype=np.byte)
    vmask = property(_getvmask,_setvmask,doc="vmask is the 2D imagePoints mask")

    @property
    def n(self):
        """n - number of points"""
        return self.__n

    @property
    def pnp(self):
        """pnp - number of parameters per world point, should be 3"""
        return self.__pnp

    @property
    def mnp(self):
        """mnp - number of parameters per image point, should be 2"""
        return self.__mnp

    @property
    def hascovx(self):
        """True if it has a covx that is not None"""
        if self.covx==None:
            return False
        else:
            return True

    @property
    def ncameras(self):
        """ncameras = m = number of parameters for 1 cameras"""
        return self.__ncameras

    def Bravel(self):
        """Use this to ravel B for combination with A in preparing sba inputs"""
        return self.B.ravel()

    def vmaskToC(self):
        """Converts vmask to C ``char *`` vmask"""
        ct = self.__vmask.ravel()
        return ct.ctypes.data_as(ctypes.c_char_p)
    
    def XtoC(self):
        """Converts X to C ``double *`` x
        Rewritted Jan 2015 by Nick Gravish to address strange bugs seen
        with large initial error; In eucsbademo.c from Lourakis, the
        points are read in this manner in readparams.c so this is correct.
        Points are raveled all packed together with the vmask holding 
        where they actually are (odd representation of sparse array done
        for reasons of Lourakis' C-ness.)
        -DE
        """
        temp = np.zeros(self.n*self.ncameras*self.mnp,dtype=np.double)
        idx = 0
        for b in range(self.n):
            for a in range(self.ncameras):
                if(self.vmask[b,a] == 1):
                    for c in range(2):
                        temp[idx] = self.__X[b,a,c]
                        idx += 1
        #logging.debug("Created temp {0},{1}".format(temp,temp.shape))
        return temp#.ctypes.data_as(ctypes.POINTER(ctypes.c_double))

    def __str__(self):
        """Provides readable representation of Points"""
        return("<sba.Points() with {0} world points (R{2}) in {1} cameras (R{3})>".format(self.n,self.ncameras,self.pnp,self.mnp))

    def _fromTxt(cls,ptsfile,ncameras):
        """Alternate constructor
        Construct array of points from sba formatted text file"""
        
        #logging.debug("pointsFromTxt() reading points from {0}".format(ptsfile))
        ifile = open(ptsfile,"r")
        blist = []
        for line in ifile:
            if line[0] != "#":
                splitted = line.split()
                blist.append([float(splitted[0]),
                              float(splitted[1]),
                              float(splitted[2])])
        ifile.close()
        b = np.array(blist,dtype=np.double)
        npoints = len(blist)

        ifile = open(ptsfile,"r")
        x = np.zeros((npoints,ncameras,2),dtype=np.double)
        vmask = np.zeros((npoints,ncameras),dtype=np.byte)
        pt = 0

        for line in ifile:
            if line[0] != "#":
                splitted = line.split()
                nvisible = int(float(splitted[3]))
                for a in range(nvisible):
                    cam = int(float(splitted[3+1+a*3]))
                    x[pt,cam,0] = float(splitted[3+2+a*3])
                    x[pt,cam,1] = float(splitted[3+3+a*3])
                    vmask[pt,cam] = 1
                pt += 1
        #print b,x,vmask
        result = cls(b,x,vmask)
        return result
    fromTxt = classmethod(_fromTxt)

    def toTxt(self,ptsoname):
        np.savetxt(ptsoname,self.B,"%4.6f")





    # added for Dylan Ray to use in Python gui version
    def _fromDylan(cls,dylan_array):
        """
        Alternate constructor to read points from a numpy array of shape
        (npoints,3+2*ncameras).

        Each row is a point + camera observations of the point.  
        The columns are as follows:
        X Y Z u1 v1 u2 v2...
        
        If a point is not observed in camera n, the value of un,vn is np.nan
        The shape of the array should be (Npoints,2*Ncameras+3)

        dtype is np.double
        """
        
        #logging.debug("sba.Points.fromDylan() reading points from numpy array")
        #logging.warning("this feature not yet tested!")

        # straight copy the points' X,Y,Z from dylan_array
        b = dylan_array[:,:3].astype(np.double)
        npoints = len(dylan_array)
        ncameras = int((dylan_array.shape[1]-3)/2)

        #ifile = file(ptsfile,"r")
        # to get x and vmask requires some reshaping
        x = np.zeros((npoints,ncameras,2),dtype=np.double)
        vmask = np.zeros((npoints,ncameras),dtype=np.byte)
        pt = 0

        for aa in range(npoints):
            for bb in range(ncameras):
                u = dylan_array[aa,3+2*bb]
                v = dylan_array[aa,3+2*bb+1]
                if (not(np.isnan(u))) and (not(np.isnan(v))):
                    x[aa,bb,0] = u
                    x[aa,bb,1] = v
                    vmask[aa,bb] = 1
                # otherwise vmask is already 0
 
        #print "b",b.shape,b
        #print "x",x.shape,x
        #print "vmask",vmask.shape,vmask

        result = cls(b,x,vmask)
        return result
    fromDylan = classmethod(_fromDylan)


    def toDylan(self):
        """
        Returns a single numpy array of shape (n,3+2*ncameras) 
        with np.nan if not visible in cameras. 
        """
        result = np.zeros((self.__n, 
                           3+self.__ncameras*2))
        result[:,:3] = self.X
        result[:,3:] = self.B
        r,c = self.vmask.shape
        for eachrow in range(self.__n):
            for eachcol in range(self.__ncameras):
                if not(self.vmask[eachrow,eachcol]):
                    result[eachrow,3+2*eachcol] = np.nan
                    result[eachrow,3+2*eachcol+1] = np.nan

        return(result)



#!/usr/bin/env python
"""
quaternions.py
(c) 2013 Dennis Evangelista
Some helper functions that do quaternion rotations
"""

from __future__ import absolute_import
import logging
import numpy as np

def quatFromArray(a):
    """
    Construct a quaternion from four components
    a = something with four numerical components
    """
    assert a[0]**2+a[1]**2+a[2]**2+a[3]**2 == 1
    return Quaternion(re=a[0],i=a[1],j=a[2],k=a[3])

def quatFromAa(aa):
    """
    Construct a unit quaternion from an axis-angle representation
    aa = something with 3 numerical components that can have norm taken
    """
    theta = np.linalg.norm(aa)
    if (theta!=0.):
        uvec = aa/theta
    else:
        uvec = np.array([0.,0.,0.])
    re = np.cos(theta/2.)
    i = uvec[0]*np.sin(theta/2.)
    j = uvec[1]*np.sin(theta/2.)
    k = uvec[2]*np.sin(theta/2.)
    return Quaternion(re,i,j,k)

def quatFromRotationMatrix(R):
    """
    Construct a unit quaternion from a rotation matrix
    """
    a = 0.5*np.sqrt(1+R[0,0]+R[1,1]+R[2,2])
    b = np.copysign(0.5*np.sqrt(1+R[0,0]-R[1,1]-R[2,2]),R[2,1]-R[1,2])
    c = np.copysign(0.5*np.sqrt(1-R[0,0]+R[1,1]-R[2,2]),R[0,2]-R[2,0])
    d = np.copysign(0.5*np.sqrt(1-R[0,0]-R[1,1]+R[2,2]),R[1,0]-R[0,1])
    return Quaternion(a,b,c,d)





    
class Quaternion():
    """
    Class and methods for handling rotations using (unit) quaternions
    """
    def __init__(self,re=1.,i=0.,j=0.,k=0.):
        """
        Constructs a quaternion.  Doesn't currently check or adjust if
        the components are not for a unit quaternion, which shouldn't be
        a problem so long as they are always constructed using fromaa or
        from a rotation matrix. 
        """
        self.re=re
        self.i=i
        self.j=j
        self.k=k
    def abs(self):
        """
        Returns magnitude (norm) of a quaternion. 
        This should always be 1??? May not be if not unit quaternion. 
        """
        return np.linalg.norm([self.re,self.i,self.j,self.k])
    def imag(self):
        """
        Returns imaginary part of a quaternion as 3 element Numpy array 
        """
        return np.array([self.i,self.j,self.k])
    def asAa(self):
        """
        Returns the quaternion in axis-angle representation, as 
        a numpy array of 3 components. This part assumes it is a 
        unit quaternion!
        """
        theta = 2.*np.arctan2(np.linalg.norm(self.imag()),self.re)
        if (theta!=0.):
            return self.imag()/np.sin(theta/2.)
        else:
            return np.array([0.,0.,0.])
    def asVector(self):
        """
        Returns quaternion as a 4 component vector numpy array. 
        """
        return np.array([self.re,self.i,self.j,self.k])
    def mul(self,q2):
        """
        Multiplication by another Quaternion q2.
        """
        s = self.re
        v = self.imag()
        t = q2.re
        w = q2.imag()
        result = Quaternion()
        result.re = s*t-v.dot(w)
        vecpart = s*w+t*v+np.cross(v,w)
        result.i = vecpart[0]
        result.j = vecpart[1]
        result.k = vecpart[2]
        return result
    def rotate(self,vec):
        """
        Rotates vector according to the Hamiltonian product q v q*
        vec = numpy array, vector with 3 components
        Returns a numpy array vector with 3 components
        """
        result = vec+np.cross(2*self.imag(),(np.cross(self.imag(),vec)+self.re*vec))
        return result
    def asRotationMatrix(self):
        a,b,c,d = self.asVector().tolist()
        return np.matrix([[a**2.+b**2.-c**2.-d**2.,2.*b*c-2.*a*d,2.*b*d+2.*a*d],
                          [2.*b*c+2.*a*d,a**2.-b**2.+c**2.-d**2.,2.*c*d-2.*a*b],
                          [2.*b*d-2.*a*c,2.*c*d+2.*a*b,a**2.-b**2.-c**2.+d**2.]])

    def __repr__(self):
        """
        For printout to screen of quaternion as R4. 
        """
        return self.asVector().__repr__()

    
